@@ -1,7 +1,7 @@
 /*
  * fsp-session.c
  *
- * Copyright (C) 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2010-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * Some parts of this file were based on source code from the libsoup
@@ -39,7 +39,7 @@
 #define FLICKR_API_UPLOAD_URL "http://api.flickr.com/services/upload"
 
 #if DEBUG_ENABLED
-#define DEBUG(...) g_debug (__VA_ARGS__);
+#define DEBUG(...) g_debug (__VA_ARGS__)
 #else
 #define DEBUG(...)
 #endif
@@ -49,7 +49,7 @@
                                 FSP_TYPE_SESSION,       \
                                 FspSessionPrivate))
 
-G_DEFINE_TYPE (FspSession, fsp_session, G_TYPE_OBJECT);
+G_DEFINE_TYPE (FspSession, fsp_session, G_TYPE_OBJECT)
 
 
 /* Private struct */
@@ -60,6 +60,7 @@ struct _FspSessionPrivate
   gchar *token;
   gchar *frob;
 
+  SoupURI *proxy_uri;
   SoupSession *soup_session;
 };
 
@@ -323,6 +324,12 @@ fsp_session_dispose                     (GObject* object)
       self->priv->soup_session = NULL;
     }
 
+  if (self->priv->proxy_uri)
+    {
+      soup_uri_free (self->priv->proxy_uri);
+      self->priv->proxy_uri = NULL;
+    }
+
   /* Call superclass */
   G_OBJECT_CLASS (fsp_session_parent_class)->dispose(object);
 }
@@ -336,6 +343,7 @@ fsp_session_finalize                    (GObject* object)
   g_free (self->priv->api_key);
   g_free (self->priv->secret);
   g_free (self->priv->token);
+  g_free (self->priv->frob);
 
   /* Call superclass */
   G_OBJECT_CLASS (fsp_session_parent_class)->finalize(object);
@@ -389,6 +397,8 @@ fsp_session_init                        (FspSession *self)
   self->priv->token = NULL;
   self->priv->frob = NULL;
 
+  self->priv->proxy_uri = NULL;
+
   self->priv->soup_session = soup_session_async_new ();
 }
 
@@ -513,11 +523,10 @@ _get_soup_message_for_upload            (GFile       *file,
   GHashTableIter iter;
   gpointer key, value;
   gchar *mime_type;
-  gchar *filepath;
   gchar *fileuri;
 
   /* Gather needed information */
-  filepath = g_file_get_path (file);
+  fileuri = g_file_get_uri (file);
   file_info = g_file_query_info (file,
                                  G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                                  G_FILE_QUERY_INFO_NONE,
@@ -537,17 +546,15 @@ _get_soup_message_for_upload            (GFile       *file,
 
   /* Append the content of the file */
   buffer = soup_buffer_new (SOUP_MEMORY_TAKE, contents, length);
-  fileuri = g_filename_to_uri (filepath, NULL, NULL);
   soup_multipart_append_form_file (mpart, "photo", fileuri,
                                    mime_type, buffer);
-  g_free (fileuri);
 
   /* Get the associated message */
   msg = soup_form_request_new_from_multipart (FLICKR_API_UPLOAD_URL, mpart);
 
   /* Free */
   soup_multipart_free (mpart);
-  g_free (filepath);
+  g_free (fileuri);
   g_free (mime_type);
 
   /* Return message */
@@ -559,10 +566,6 @@ _load_file_contents_cb                  (GObject      *object,
                                          GAsyncResult *res,
                                          gpointer      data)
 {
-  g_return_if_fail (G_IS_FILE (object));
-  g_return_if_fail (G_IS_ASYNC_RESULT (res));
-  g_return_if_fail (data != NULL);
-
   UploadPhotoData *up_clos = NULL;
   AsyncRequestData *ard_clos = NULL;
   GHashTable *extra_params = NULL;
@@ -571,6 +574,10 @@ _load_file_contents_cb                  (GObject      *object,
   gchar *contents;
   gsize length;
 
+  g_return_if_fail (G_IS_FILE (object));
+  g_return_if_fail (G_IS_ASYNC_RESULT (res));
+  g_return_if_fail (data != NULL);
+
   /* Get data from UploadPhotoData closure, and free it */
   up_clos = (UploadPhotoData *) data;
   ard_clos = up_clos->ar_data;
@@ -658,11 +665,11 @@ static GHashTable *
 _get_params_table_from_valist           (const gchar *first_param,
                                          va_list      args)
 {
-  g_return_val_if_fail (first_param != NULL, NULL);
-  g_return_val_if_fail (args != NULL, NULL);
-
   GHashTable *table = NULL;
-  gchar *p, *v;
+  gchar *p = NULL;
+  gchar *v = NULL;
+
+  g_return_val_if_fail (first_param != NULL, NULL);
 
   table = g_hash_table_new_full (g_str_hash, g_str_equal,
                                  (GDestroyNotify)g_free,
@@ -711,12 +718,12 @@ static gchar *
 _get_signed_query_with_params           (const gchar      *api_sig,
                                          GHashTable       *params_table)
 {
-  g_return_val_if_fail (params_table != NULL, NULL);
-  g_return_val_if_fail (api_sig != NULL, NULL);
-
   GList *keys = NULL;
   gchar *retval = NULL;
 
+  g_return_val_if_fail (params_table != NULL, NULL);
+  g_return_val_if_fail (api_sig != NULL, NULL);
+
   /* Get ownership of the table */
   g_hash_table_ref (params_table);
 
@@ -790,10 +797,10 @@ static gboolean
 _check_errors_on_soup_response           (SoupMessage  *msg,
                                           GError      **error)
 {
-  g_assert (SOUP_IS_MESSAGE (msg));
-
   GError *err = NULL;
 
+  g_assert (SOUP_IS_MESSAGE (msg));
+
   /* Check non-succesful SoupMessage's only */
   if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
     {
@@ -825,11 +832,11 @@ _check_async_errors_on_finish           (GObject       *object,
                                          gpointer       source_tag,
                                          GError       **error)
 {
+  gboolean errors_found = TRUE;
+
   g_return_val_if_fail (G_IS_OBJECT (object), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  gboolean errors_found = TRUE;
-
   if (g_simple_async_result_is_valid (res, object, source_tag))
     {
       GSimpleAsyncResult *simple = NULL;
@@ -849,12 +856,12 @@ gchar *
 _get_api_signature_from_hash_table      (const gchar *shared_secret,
                                          GHashTable  *params_table)
 {
-  g_return_val_if_fail (shared_secret != NULL, NULL);
-  g_return_val_if_fail (params_table != NULL, NULL);
-
   GList *keys = NULL;
   gchar *api_sig = NULL;
 
+  g_return_val_if_fail (shared_secret != NULL, NULL);
+  g_return_val_if_fail (params_table != NULL, NULL);
+
   /* Get ownership of the table */
   g_hash_table_ref (params_table);
 
@@ -905,14 +912,14 @@ _get_signed_query                       (const gchar *shared_secret,
                                          const gchar *first_param,
                                          ... )
 {
-  g_return_val_if_fail (shared_secret != NULL, NULL);
-  g_return_val_if_fail (first_param != NULL, NULL);
-
   va_list args;
   GHashTable *table = NULL;
   gchar *api_sig = NULL;
   gchar *retval = NULL;
 
+  g_return_val_if_fail (shared_secret != NULL, NULL);
+  g_return_val_if_fail (first_param != NULL, NULL);
+
   va_start (args, first_param);
 
   /* Get the hash table for the params and the API signature from it */
@@ -941,14 +948,14 @@ _perform_async_request                  (SoupSession         *soup_session,
                                          gpointer             source_tag,
                                          gpointer             data)
 {
+  AsyncRequestData *clos = NULL;
+  SoupMessage *msg = NULL;
+
   g_return_if_fail (SOUP_IS_SESSION (soup_session));
   g_return_if_fail (url != NULL);
   g_return_if_fail (request_cb != NULL);
   g_return_if_fail (callback != NULL);
 
-  AsyncRequestData *clos = NULL;
-  SoupMessage *msg = NULL;
-
   /* Build and queue the message */
   msg = soup_message_new (SOUP_METHOD_GET, url);
 
@@ -989,14 +996,12 @@ _soup_session_cancelled_cb              (GCancellable *cancellable,
                                          gpointer      data)
 {
   AsyncRequestData *clos = NULL;
-  GObject *object = NULL;
   SoupSession *session = NULL;
   SoupMessage *message = NULL;
 
   clos = (AsyncRequestData *) data;
 
   /* Get data from closure, and free it */
-  object = clos->object;
   session = clos->soup_session;
   message = clos->soup_message;
 
@@ -1010,10 +1015,6 @@ _handle_soup_response                   (SoupMessage   *msg,
                                          FspParserFunc  parserFunc,
                                          gpointer       data)
 {
-  g_assert (SOUP_IS_MESSAGE (msg));
-  g_assert (parserFunc != NULL);
-  g_assert (data != NULL);
-
   FspParser *parser = NULL;
   AsyncRequestData *clos = NULL;
   gpointer result = NULL;
@@ -1021,6 +1022,10 @@ _handle_soup_response                   (SoupMessage   *msg,
   gchar *response_str = NULL;
   gulong response_len = 0;
 
+  g_assert (SOUP_IS_MESSAGE (msg));
+  g_assert (parserFunc != NULL);
+  g_assert (data != NULL);
+
   parser = fsp_parser_get_instance ();
   clos = (AsyncRequestData *) data;
 
@@ -1028,7 +1033,7 @@ _handle_soup_response                   (SoupMessage   *msg,
   g_signal_handlers_disconnect_by_func (msg, _wrote_body_data_cb, clos);
 
   response_str = g_strndup (msg->response_body->data, msg->response_body->length);
-  response_len = (ulong) msg->response_body->length;
+  response_len = (gulong) msg->response_body->length;
   if (response_str)
     DEBUG ("\nResponse got:\n%s\n", response_str);
 
@@ -1048,8 +1053,6 @@ _build_async_result_and_complete        (AsyncRequestData *clos,
                                          gpointer    result,
                                          GError     *error)
 {
-  g_assert (clos != NULL);
-
   GSimpleAsyncResult *res = NULL;
   GObject *object = NULL;
   GCancellableData *cancellable_data = NULL;
@@ -1059,6 +1062,8 @@ _build_async_result_and_complete        (AsyncRequestData *clos,
   gpointer source_tag;
   gpointer data;
 
+  g_assert (clos != NULL);
+
   /* Get data from closure, and free it */
   object = clos->object;
   cancellable = clos->cancellable;
@@ -1096,11 +1101,11 @@ _finish_async_request                   (GObject       *object,
                                          gpointer       source_tag,
                                          GError       **error)
 {
+  gpointer retval = NULL;
+
   g_return_val_if_fail (G_IS_OBJECT (object), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  gpointer retval = NULL;
-
   /* Check for errors */
   if (!_check_async_errors_on_finish (object, res, source_tag, error))
     {
@@ -1125,11 +1130,11 @@ _photo_upload_soup_session_cb           (SoupSession *session,
                                          SoupMessage *msg,
                                          gpointer     data)
 {
+  AsyncRequestData *ard_clos = NULL;
+
   g_assert (SOUP_IS_MESSAGE (msg));
   g_assert (data != NULL);
 
-  AsyncRequestData *ard_clos = NULL;
-
   ard_clos = (AsyncRequestData *) data;
   g_signal_handlers_disconnect_by_func (msg, _wrote_body_data_cb, ard_clos->object);
 
@@ -1249,22 +1254,22 @@ fsp_session_new                         (const gchar *api_key,
   g_return_val_if_fail (api_key != NULL, NULL);
   g_return_val_if_fail (secret != NULL, NULL);
 
-  GObject *object = g_object_new(FSP_TYPE_SESSION,
-                                 "api-key", api_key,
-                                 "secret", secret,
-                                 "token", token,
-                                 NULL);
-  return FSP_SESSION (object);
+  return FSP_SESSION (g_object_new(FSP_TYPE_SESSION,
+                                   "api-key", api_key,
+                                   "secret", secret,
+                                   "token", token,
+                                   NULL));
 }
 
-void
+gboolean
 fsp_session_set_http_proxy              (FspSession *self,
                                          const char *host, const char *port,
                                          const char *username, const char *password)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-
   SoupURI *proxy_uri = NULL;
+
+  g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
+
   if (host != NULL)
     {
       const gchar *actual_user = NULL;
@@ -1295,11 +1300,28 @@ fsp_session_set_http_proxy              (FspSession *self,
       soup_uri_set_password (proxy_uri, actual_password);
     }
 
-  /* Set/unset the proxy */
-  g_object_set (G_OBJECT (self->priv->soup_session),
-                SOUP_SESSION_PROXY_URI,
-                proxy_uri,
-                NULL);
+  /* Set/unset the proxy if needed */
+  if ((!self->priv->proxy_uri && proxy_uri)
+      || (self->priv->proxy_uri && !proxy_uri)
+      || (self->priv->proxy_uri && proxy_uri && !soup_uri_equal (self->priv->proxy_uri, proxy_uri)))
+    {
+      g_object_set (G_OBJECT (self->priv->soup_session),
+                    SOUP_SESSION_PROXY_URI,
+                    proxy_uri,
+                    NULL);
+
+      /* Save internal reference to the proxy */
+      if (self->priv->proxy_uri)
+        soup_uri_free (self->priv->proxy_uri);
+
+      self->priv->proxy_uri = proxy_uri;
+
+      /* Proxy configuration actually changed */
+      return TRUE;
+    }
+
+  /* Proxy configuration has not changed */
+  return FALSE;
 }
 
 const gchar *
@@ -1344,13 +1366,14 @@ fsp_session_get_auth_url_async          (FspSession          *self,
                                          GAsyncReadyCallback  cb,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.auth.getFrob",
                                     "api_key", priv->api_key,
@@ -1371,15 +1394,14 @@ fsp_session_get_auth_url_finish         (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
-  g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
-  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
-
   gchar *frob = NULL;
   gchar *auth_url = NULL;
 
+  g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
+  g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
+
   frob = (gchar*) _finish_async_request (G_OBJECT (self), res,
                                          fsp_session_get_auth_url_async, error);
-
   /* Build the auth URL from the frob */
   if (frob != NULL)
     {
@@ -1387,6 +1409,7 @@ fsp_session_get_auth_url_finish         (FspSession    *self,
       gchar *signed_query = NULL;
 
       /* Save the frob */
+      g_free (priv->frob);
       priv->frob = frob;
 
       /* Build the authorization url */
@@ -1411,11 +1434,12 @@ fsp_session_complete_auth_async         (FspSession          *self,
                                          GAsyncReadyCallback  cb,
                                          gpointer             data)
 {
+  FspSessionPrivate *priv = NULL;
+
   g_return_if_fail (FSP_IS_SESSION (self));
   g_return_if_fail (cb != NULL);
 
-  FspSessionPrivate *priv = self->priv;
-
+  priv = self->priv;
   if (priv->frob != NULL)
     {
       gchar *signed_query = NULL;
@@ -1453,11 +1477,11 @@ fsp_session_complete_auth_finish        (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  FspDataAuthToken *auth_token = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  FspDataAuthToken *auth_token = NULL;
-
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
                                                 fsp_session_complete_auth_async,
@@ -1476,11 +1500,12 @@ fsp_session_check_auth_info_async       (FspSession          *self,
                                          GAsyncReadyCallback  cb,
                                          gpointer             data)
 {
+  FspSessionPrivate *priv = NULL;
+
   g_return_if_fail (FSP_IS_SESSION (self));
   g_return_if_fail (cb != NULL);
 
-  FspSessionPrivate *priv = self->priv;
-
+  priv = self->priv;
   if (priv->token != NULL)
     {
       gchar *signed_query = NULL;
@@ -1518,11 +1543,11 @@ fsp_session_check_auth_info_finish      (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  FspDataAuthToken *auth_token = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  FspDataAuthToken *auth_token = NULL;
-
   auth_token =
     FSP_DATA_AUTH_TOKEN (_finish_async_request (G_OBJECT (self), res,
                                                 fsp_session_check_auth_info_async,
@@ -1536,11 +1561,12 @@ fsp_session_get_upload_status_async     (FspSession          *self,
                                          GAsyncReadyCallback cb,
                                          gpointer             data)
 {
+  FspSessionPrivate *priv = NULL;
+
   g_return_if_fail (FSP_IS_SESSION (self));
   g_return_if_fail (cb != NULL);
 
-  FspSessionPrivate *priv = self->priv;
-
+  priv = self->priv;
   if (priv->token != NULL)
     {
       gchar *signed_query = NULL;
@@ -1578,11 +1604,11 @@ fsp_session_get_upload_status_finish    (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  FspDataUploadStatus *upload_status = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  FspDataUploadStatus *upload_status = NULL;
-
   upload_status =
     FSP_DATA_UPLOAD_STATUS (_finish_async_request (G_OBJECT (self), res,
                                                    fsp_session_get_upload_status_async,
@@ -1592,7 +1618,7 @@ fsp_session_get_upload_status_finish    (FspSession    *self,
 
 void
 fsp_session_upload_async                (FspSession          *self,
-                                         const gchar         *filepath,
+                                         const gchar         *fileuri,
                                          const gchar         *title,
                                          const gchar         *description,
                                          const gchar         *tags,
@@ -1606,9 +1632,6 @@ fsp_session_upload_async                (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-  g_return_if_fail (filepath != NULL);
-
   FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   GHashTable *extra_params = NULL;
@@ -1617,6 +1640,9 @@ fsp_session_upload_async                (FspSession          *self,
   GFile *file = NULL;
   gchar *api_sig = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (fileuri != NULL);
+
   /* Get flickr proxy and extra params (those actually used) */
   priv = self->priv;
   soup_session = _get_soup_session (self);
@@ -1650,7 +1676,7 @@ fsp_session_upload_async                (FspSession          *self,
   up_clos->extra_params = extra_params;
 
   /* Asynchronously load the contents of the file */
-  file = g_file_new_for_path (filepath);
+  file = g_file_new_for_uri (fileuri);
   g_file_load_contents_async (file, NULL, _load_file_contents_cb, up_clos);
 }
 
@@ -1662,10 +1688,9 @@ fsp_session_upload_finish               (FspSession    *self,
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  gchar *photo_id = (gchar*) _finish_async_request (G_OBJECT (self), res,
-                                                    fsp_session_upload_async,
-                                                    error);
-  return photo_id;
+  return (gchar*) _finish_async_request (G_OBJECT (self), res,
+                                         fsp_session_upload_async,
+                                         error);
 }
 
 void
@@ -1675,15 +1700,16 @@ fsp_session_get_info_async              (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-  g_return_if_fail (photo_id != NULL);
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (photo_id != NULL);
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.photos.getInfo",
                                     "api_key", priv->api_key,
@@ -1708,11 +1734,11 @@ fsp_session_get_info_finish             (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  FspDataPhotoInfo *photo_info = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  FspDataPhotoInfo *photo_info = NULL;
-
   photo_info =
     FSP_DATA_PHOTO_INFO (_finish_async_request (G_OBJECT (self), res,
                                                 fsp_session_get_info_async,
@@ -1726,14 +1752,15 @@ fsp_session_get_photosets_async         (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.photosets.getList",
                                     "api_key", priv->api_key,
@@ -1760,12 +1787,9 @@ fsp_session_get_photosets_finish        (FspSession    *self,
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  GSList *photosets_list = NULL;
-
-  photosets_list = (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                                    fsp_session_get_photosets_async,
-                                                    error);
-  return photosets_list;
+  return (GSList*) _finish_async_request (G_OBJECT (self), res,
+                                          fsp_session_get_photosets_async,
+                                          error);
 }
 
 void
@@ -1776,16 +1800,17 @@ fsp_session_add_to_photoset_async       (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-  g_return_if_fail (photo_id != NULL);
-  g_return_if_fail (photoset_id != NULL);
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (photo_id != NULL);
+  g_return_if_fail (photoset_id != NULL);
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.photosets.addPhoto",
                                     "api_key", priv->api_key,
@@ -1811,11 +1836,11 @@ fsp_session_add_to_photoset_finish      (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  gpointer result = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  gpointer result = NULL;
-
   result = _finish_async_request (G_OBJECT (self), res,
                                   fsp_session_add_to_photoset_async, error);
 
@@ -1831,16 +1856,17 @@ fsp_session_create_photoset_async       (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-  g_return_if_fail (title != NULL);
-  g_return_if_fail (primary_photo_id != NULL);
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (title != NULL);
+  g_return_if_fail (primary_photo_id != NULL);
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.photosets.create",
                                     "api_key", priv->api_key,
@@ -1855,7 +1881,7 @@ fsp_session_create_photoset_async       (FspSession          *self,
 
   /* Perform the async request */
   soup_session = _get_soup_session (self);
-  _perform_async_request (priv->soup_session, url,
+  _perform_async_request (soup_session, url,
                           _create_photoset_soup_session_cb, G_OBJECT (self),
                           cancellable, callback, fsp_session_create_photoset_async, data);
 
@@ -1867,11 +1893,11 @@ fsp_session_create_photoset_finish      (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  gchar *photoset_id = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  gchar *photoset_id = NULL;
-
   photoset_id =
     (gchar*) _finish_async_request (G_OBJECT (self), res,
                                     fsp_session_create_photoset_async,
@@ -1885,14 +1911,15 @@ fsp_session_get_groups_async            (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.groups.pools.getGroups",
                                     "api_key", priv->api_key,
@@ -1919,12 +1946,9 @@ fsp_session_get_groups_finish           (FspSession    *self,
   g_return_val_if_fail (FSP_IS_SESSION (self), NULL);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), NULL);
 
-  GSList *groups_list = NULL;
-
-  groups_list = (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                                 fsp_session_get_groups_async,
-                                                 error);
-  return groups_list;
+  return (GSList*) _finish_async_request (G_OBJECT (self), res,
+                                          fsp_session_get_groups_async,
+                                          error);
 }
 
 void
@@ -1935,16 +1959,17 @@ fsp_session_add_to_group_async          (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-  g_return_if_fail (photo_id != NULL);
-  g_return_if_fail (group_id != NULL);
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+  g_return_if_fail (photo_id != NULL);
+  g_return_if_fail (group_id != NULL);
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.groups.pools.add",
                                     "api_key", priv->api_key,
@@ -1970,11 +1995,11 @@ fsp_session_add_to_group_finish         (FspSession    *self,
                                          GAsyncResult  *res,
                                          GError       **error)
 {
+  gpointer result = NULL;
+
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  gpointer result = NULL;
-
   result = _finish_async_request (G_OBJECT (self), res,
                                   fsp_session_add_to_group_async, error);
 
@@ -1987,14 +2012,15 @@ fsp_session_get_tags_list_async         (FspSession          *self,
                                          GAsyncReadyCallback  callback,
                                          gpointer             data)
 {
-  g_return_if_fail (FSP_IS_SESSION (self));
-
-  FspSessionPrivate *priv = self->priv;
+  FspSessionPrivate *priv = NULL;
   SoupSession *soup_session = NULL;
   gchar *url = NULL;
   gchar *signed_query = NULL;
 
+  g_return_if_fail (FSP_IS_SESSION (self));
+
   /* Build the signed url */
+  priv = self->priv;
   signed_query = _get_signed_query (priv->secret,
                                     "method", "flickr.tags.getListUser",
                                     "api_key", priv->api_key,
@@ -2021,10 +2047,6 @@ fsp_session_get_tags_list_finish        (FspSession    *self,
   g_return_val_if_fail (FSP_IS_SESSION (self), FALSE);
   g_return_val_if_fail (G_IS_ASYNC_RESULT (res), FALSE);
 
-  GSList *tags_list = NULL;
-
-  tags_list = (GSList*) _finish_async_request (G_OBJECT (self), res,
-                                               fsp_session_get_tags_list_async, error);
-
-  return tags_list;
+  return (GSList*) _finish_async_request (G_OBJECT (self), res,
+                                          fsp_session_get_tags_list_async, error);
 }
