@@ -1,7 +1,7 @@
 /*
  * fsp-session.h
  *
- * Copyright (C) 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2010-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -67,7 +67,7 @@ FspSession *
 fsp_session_new                         (const gchar *api_key,
                                          const gchar *secret,
                                          const gchar *token);
-void
+gboolean
 fsp_session_set_http_proxy              (FspSession *self,
                                          const char *host, const char *port,
                                          const char *username, const char *password);
@@ -127,7 +127,7 @@ fsp_session_get_upload_status_finish    (FspSession    *self,
 
 void
 fsp_session_upload_async                (FspSession          *self,
-                                         const gchar         *filepath,
+                                         const gchar         *fileuri,
                                          const gchar         *title,
                                          const gchar         *description,
                                          const gchar         *tags,
