@@ -1,7 +1,7 @@
 /*
  * frogr-add-tags-dialog.c -- Frogr 'add tags' dialog
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -22,6 +22,7 @@
 
 #include "frogr-add-tags-dialog.h"
 
+#include "frogr-config.h"
 #include "frogr-global-defs.h"
 #include "frogr-picture.h"
 
@@ -33,7 +34,7 @@
                                 FROGR_TYPE_ADD_TAGS_DIALOG,     \
                                 FrogrAddTagsDialogPrivate))
 
-G_DEFINE_TYPE (FrogrAddTagsDialog, frogr_add_tags_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (FrogrAddTagsDialog, frogr_add_tags_dialog, GTK_TYPE_DIALOG)
 
 typedef struct _FrogrAddTagsDialogPrivate {
   GtkWidget *entry;
@@ -95,7 +96,6 @@ _tag_list_completion_func (GtkEntryCompletion *completion, const gchar *key,
 {
   FrogrAddTagsDialog *self = NULL;
   FrogrAddTagsDialogPrivate *priv = NULL;
-  const gchar *entry_text = NULL;
   gchar *stripped_entry_text = NULL;
   gchar *basetext = NULL;
   gchar *tag = NULL;
@@ -118,7 +118,6 @@ _tag_list_completion_func (GtkEntryCompletion *completion, const gchar *key,
     return FALSE;
 
   /* Look for the last token in 'key' */
-  entry_text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
   stripped_entry_text = gtk_editable_get_chars (GTK_EDITABLE (priv->entry), 0, cursor_pos);
   stripped_entry_text = g_strstrip (stripped_entry_text);
   basetext = g_strrstr (stripped_entry_text, " ");
@@ -156,7 +155,7 @@ _completion_match_selected_cb (GtkEntryCompletion *widget, GtkTreeModel *model,
   glong matching_text_len = 0;
 
   self = FROGR_ADD_TAGS_DIALOG (data);
-  priv = FROGR_ADD_TAGS_DIALOG_GET_PRIVATE (data);
+  priv = FROGR_ADD_TAGS_DIALOG_GET_PRIVATE (self);
   gtk_tree_model_get (model, iter, TEXT_COL, &tag, -1);
 
   entry_text = gtk_entry_get_text (GTK_ENTRY (priv->entry));
@@ -203,12 +202,10 @@ _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data)
         {
           FrogrPicture *picture;
           GSList *item;
-          guint n_pictures;
 
           DEBUG ("Adding tags to picture(s): %s", tags);
 
           /* Iterate over the rest of elements */
-          n_pictures = g_slist_length (priv->pictures);
           for (item = priv->pictures; item; item = g_slist_next (item))
             {
               picture = FROGR_PICTURE (item->data);
@@ -321,10 +318,10 @@ static void
 frogr_add_tags_dialog_init (FrogrAddTagsDialog *self)
 {
   FrogrAddTagsDialogPrivate *priv = FROGR_ADD_TAGS_DIALOG_GET_PRIVATE (self);
+  FrogrConfig *config = NULL;
   GtkWidget *vbox = NULL;
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
-  GtkEntryCompletion *completion = NULL;
   GtkTreeModel *model = NULL;
 
   /* Create widgets */
@@ -344,31 +341,34 @@ frogr_add_tags_dialog_init (FrogrAddTagsDialog *self)
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
 
-  completion = gtk_entry_completion_new ();
-  gtk_entry_completion_set_text_column (GTK_ENTRY_COMPLETION (completion), TEXT_COL);
-  gtk_entry_completion_set_inline_completion (GTK_ENTRY_COMPLETION (completion), TRUE);
-  gtk_entry_completion_set_match_func (GTK_ENTRY_COMPLETION (completion),
-                                       _tag_list_completion_func,
-                                       self, NULL);
-
   model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
-  gtk_entry_completion_set_model (GTK_ENTRY_COMPLETION (completion), model);
   priv->treemodel = model;
-
   priv->entry = gtk_entry_new ();
-  gtk_entry_set_completion (GTK_ENTRY (priv->entry), completion);
+
+  config = frogr_config_get_instance ();
+  if (config && frogr_config_get_tags_autocompletion (config))
+    {
+      GtkEntryCompletion *completion = NULL;
+      completion = gtk_entry_completion_new ();
+      gtk_entry_completion_set_text_column (GTK_ENTRY_COMPLETION (completion), TEXT_COL);
+      gtk_entry_completion_set_inline_completion (GTK_ENTRY_COMPLETION (completion), TRUE);
+      gtk_entry_completion_set_match_func (GTK_ENTRY_COMPLETION (completion),
+                                           _tag_list_completion_func,
+                                           self, NULL);
+
+      gtk_entry_completion_set_model (GTK_ENTRY_COMPLETION (completion), model);
+
+      gtk_entry_set_completion (GTK_ENTRY (priv->entry), completion);
+
+      g_signal_connect (G_OBJECT (completion), "match-selected",
+                        G_CALLBACK (_completion_match_selected_cb), self);
+    }
 
   gtk_box_pack_start (GTK_BOX (vbox), priv->entry, TRUE, FALSE, 6);
   gtk_widget_set_size_request (GTK_WIDGET (self), 300, -1);
 
-  g_signal_connect (G_OBJECT (completion), "match-selected",
-                    G_CALLBACK (_completion_match_selected_cb), self);
-
   g_signal_connect (G_OBJECT (self), "response",
                     G_CALLBACK (_dialog_response_cb), NULL);
-
-  /* Show the UI */
-  gtk_widget_show_all (GTK_WIDGET (self));
 }
 
 /* Public API */
