@@ -1,7 +1,7 @@
 /*
  * frogr-auth-dialog.c -- Authorization dialog
  *
- * Copyright (C) 2009, 2010 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -23,13 +23,13 @@
 #include "frogr-auth-dialog.h"
 
 #include "frogr-controller.h"
+#include "frogr-global-defs.h"
 
 #include <config.h>
 #include <glib/gi18n.h>
 
 static gchar *unauth_txt =
-  N_("You have not properly authorized %s in flickr.\n"
-     "Please press the button to authorize %s "
+  N_("Please press the button to authorize %s "
      "and then come back to this screen to complete the process.");
 
 static gchar *auth_txt =
@@ -52,13 +52,18 @@ static void
 _ask_for_authorization (GtkWindow *parent)
 {
   GtkWidget *dialog = NULL;
+  gchar *title = NULL;
+
   dialog = gtk_message_dialog_new (parent,
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_INFO,
                                    GTK_BUTTONS_OK,
                                    _(unauth_txt),
-                                   PACKAGE_NAME,
-                                   PACKAGE_NAME);
+                                   APP_SHORTNAME);
+
+  title = g_strdup_printf ("Authorize %s", APP_SHORTNAME);
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  g_free (title);
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (_ask_for_authorization_response_cb), NULL);
@@ -82,12 +87,18 @@ static void
 _ask_for_auth_confirmation (GtkWindow *parent)
 {
   GtkWidget *dialog = NULL;
+  gchar *title = NULL;
+
   dialog = gtk_message_dialog_new (parent,
                                    GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                    GTK_MESSAGE_INFO,
                                    GTK_BUTTONS_OK,
                                    _(auth_txt),
-                                   PACKAGE_NAME);
+                                   APP_SHORTNAME);
+
+  title = g_strdup_printf ("Authorize %s", APP_SHORTNAME);
+  gtk_window_set_title (GTK_WINDOW (dialog), title);
+  g_free (title);
 
   g_signal_connect (G_OBJECT (dialog), "response",
                     G_CALLBACK (_ask_for_auth_confirmation_response_cb), NULL);
