@@ -1,7 +1,8 @@
 /*
  * frogr-config.h -- Configuration system for Frogr.
  *
- * Copyright (C) 2009, 2010, 2011 Adrian Perez
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
+ *           (C) 2009 Adrian Perez
  * Authors:
  *   Adrian Perez <aperez@igalia.com>
  *   Mario Sanchez Prada <msanchez@igalia.com>
@@ -56,6 +57,12 @@ struct _FrogrConfigClass
 
 GType frogr_config_get_type (void) G_GNUC_CONST;
 
+typedef enum {
+  SORT_AS_LOADED,
+  SORT_BY_TITLE,
+  SORT_BY_DATE
+} SortingCriteria;
+
 FrogrConfig* frogr_config_get_instance (void);
 
 gboolean frogr_config_save_all (FrogrConfig *self);
@@ -69,7 +76,7 @@ gboolean frogr_config_add_account (FrogrConfig  *self,
 
 GSList *frogr_config_get_accounts (FrogrConfig *self);
 
-void frogr_config_set_active_account (FrogrConfig *self, const gchar *id);
+gboolean frogr_config_set_active_account (FrogrConfig *self, const gchar *id);
 
 FrogrAccount *frogr_config_get_active_account (FrogrConfig *self);
 
@@ -101,6 +108,27 @@ void frogr_config_set_default_show_in_search (FrogrConfig *self, gboolean value)
 
 gboolean frogr_config_get_default_show_in_search (FrogrConfig *self);
 
+void frogr_config_set_tags_autocompletion (FrogrConfig *self, gboolean value);
+
+gboolean frogr_config_get_tags_autocompletion (FrogrConfig *self);
+
+void frogr_config_set_keep_file_extensions (FrogrConfig *self, gboolean value);
+
+gboolean frogr_config_get_keep_file_extensions (FrogrConfig *self);
+
+void frogr_config_set_mainview_enable_tooltips (FrogrConfig *self, gboolean value);
+
+gboolean frogr_config_get_mainview_enable_tooltips (FrogrConfig *self);
+
+void frogr_config_set_mainview_sorting_criteria (FrogrConfig *self,
+                                                 SortingCriteria criteria);
+
+SortingCriteria frogr_config_get_mainview_sorting_criteria (FrogrConfig *self);
+
+void frogr_config_set_mainview_sorting_reversed (FrogrConfig *self, gboolean reversed);
+
+gboolean frogr_config_get_mainview_sorting_reversed (FrogrConfig *self);
+
 void frogr_config_set_use_proxy (FrogrConfig *self, gboolean value);
 
 gboolean frogr_config_get_use_proxy (FrogrConfig *self);
