@@ -1,7 +1,7 @@
 /*
  * frogr-controller.c -- Controller of the whole application
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -46,14 +46,14 @@
 
 #define API_KEY "18861766601de84f0921ce6be729f925"
 #define SHARED_SECRET "6233fbefd85f733a"
-#define DEFAULT_TIMEOUT 100
+#define DEFAULT_TIMEOUT 50
 
 #define FROGR_CONTROLLER_GET_PRIVATE(object)                    \
   (G_TYPE_INSTANCE_GET_PRIVATE ((object),                       \
                                 FROGR_TYPE_CONTROLLER,          \
                                 FrogrControllerPrivate))
 
-G_DEFINE_TYPE (FrogrController, frogr_controller, G_TYPE_OBJECT);
+G_DEFINE_TYPE (FrogrController, frogr_controller, G_TYPE_OBJECT)
 
 /* Private data */
 
@@ -80,8 +80,8 @@ struct _FrogrControllerPrivate
   gboolean adding_to_set;
   gboolean adding_to_group;
 
-  /* We use these to know when an empty list of tags means that the
-     user has no tags at all, as fetching already happened before */
+  gboolean account_info_fetched;
+  gboolean account_extra_info_fetched;
   gboolean sets_fetched;
   gboolean groups_fetched;
   gboolean tags_fetched;
@@ -114,6 +114,14 @@ typedef struct {
   GError *error;
 } upload_picture_st;
 
+typedef enum {
+  FETCHING_NOTHING,
+  FETCHING_ACCOUNT_INFO,
+  FETCHING_ACCOUNT_EXTRA_INFO,
+  FETCHING_SETS,
+  FETCHING_GROUPS,
+  FETCHING_TAGS
+} FetchingActivity;
 
 /* Prototypes */
 
@@ -175,7 +183,7 @@ static void _on_picture_uploaded (FrogrController *self, FrogrPicture *picture);
 static void _on_pictures_uploaded (FrogrController *self,
                                    GError *error);
 
-static void _fetch_everything (FrogrController *self);
+static void _fetch_everything (FrogrController *self, gboolean force_fetch);
 
 static void _fetch_sets (FrogrController *self);
 
@@ -197,6 +205,8 @@ static void _fetch_tags (FrogrController *self);
 
 static void _fetch_tags_cb (GObject *object, GAsyncResult *res, gpointer data);
 
+static gboolean _show_progress_on_idle (gpointer data);
+
 static gboolean _show_details_dialog_on_idle (GSList *pictures);
 
 static gboolean _show_add_tags_dialog_on_idle (GSList *pictures);
@@ -422,13 +432,11 @@ _complete_auth_cb (GObject *object, GAsyncResult *result, gpointer data)
 {
   FspSession *session = NULL;
   FrogrController *controller = NULL;
-  FrogrControllerPrivate *priv = NULL;
   FspDataAuthToken *auth_token = NULL;
   GError *error = NULL;
 
   session = FSP_SESSION (object);
   controller = FROGR_CONTROLLER (data);
-  priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
 
   auth_token = fsp_session_complete_auth_finish (session, result, &error);
   if (auth_token)
@@ -465,9 +473,6 @@ _upload_picture (FrogrController *self, FrogrPicture *picture,
                  FrogrPictureUploadedCallback picture_uploaded_cb,
                  GObject *object)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-  g_return_if_fail(FROGR_IS_PICTURE (picture));
-
   FrogrControllerPrivate *priv = NULL;
   upload_picture_st *up_st = NULL;
   FspVisibility public_visibility = FSP_VISIBILITY_NONE;
@@ -477,6 +482,9 @@ _upload_picture (FrogrController *self, FrogrPicture *picture,
   FspContentType content_type = FSP_CONTENT_TYPE_NONE;
   FspSearchScope search_scope = FSP_SEARCH_SCOPE_NONE;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+  g_return_if_fail(FROGR_IS_PICTURE (picture));
+
   up_st = g_slice_new0 (upload_picture_st);
   up_st->controller = self;
   up_st->picture = picture;
@@ -504,7 +512,7 @@ _upload_picture (FrogrController *self, FrogrPicture *picture,
 
   _enable_cancellable (self, TRUE);
   fsp_session_upload_async (priv->session,
-                            frogr_picture_get_filepath (picture),
+                            frogr_picture_get_fileuri (picture),
                             frogr_picture_get_title (picture),
                             frogr_picture_get_description (picture),
                             frogr_picture_get_tags (picture),
@@ -957,12 +965,12 @@ _notify_adding_to_group (FrogrController *self,
 static void
 _on_picture_loaded (FrogrController *self, FrogrPicture *picture)
 {
-  g_return_if_fail (FROGR_IS_CONTROLLER (self));
-  g_return_if_fail (FROGR_IS_PICTURE (picture));
-
   FrogrControllerPrivate *priv = NULL;
   FrogrMainViewModel *mainview_model = NULL;
 
+  g_return_if_fail (FROGR_IS_CONTROLLER (self));
+  g_return_if_fail (FROGR_IS_PICTURE (picture));
+
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   mainview_model = frogr_main_view_get_model (priv->mainview);
 
@@ -1000,10 +1008,7 @@ _on_pictures_uploaded (FrogrController *self,
 
   if (!error)
     {
-      FrogrControllerPrivate *priv = NULL;
-      priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-
-      /* Fetch sets and tags right after finishing */
+      /* Fetch sets and tags (if needed) right after finishing */
       _fetch_sets (self);
       _fetch_tags (self);
 
@@ -1022,28 +1027,45 @@ _on_pictures_uploaded (FrogrController *self,
 }
 
 static void
-_fetch_everything (FrogrController *self)
+_fetch_everything (FrogrController *self, gboolean force_fetch)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
-  _fetch_account_info (self);
-  _fetch_account_extra_info (self);
-  _fetch_sets (self);
-  _fetch_groups (self);
-  _fetch_tags (self);
-}
-
-static void
-_fetch_sets (FrogrController *self)
-{
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   if (!frogr_controller_is_authorized (self))
     return;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+
+  if (force_fetch || !priv->account_info_fetched)
+    _fetch_account_info (self);
+
+  if (force_fetch || !priv->account_extra_info_fetched)
+    _fetch_account_extra_info (self);
+
+  if (force_fetch || !priv->sets_fetched)
+    _fetch_sets (self);
+
+  if (force_fetch || !priv->groups_fetched)
+    _fetch_groups (self);
+
+  if (force_fetch || !priv->tags_fetched)
+    _fetch_tags (self);
+}
+
+static void
+_fetch_sets (FrogrController *self)
+{
+  FrogrControllerPrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
+  if (!frogr_controller_is_authorized (self))
+    return;
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv->sets_fetched = FALSE;
   priv->fetching_sets = TRUE;
 
   _enable_cancellable (self, TRUE);
@@ -1117,14 +1139,15 @@ _fetch_sets_cb (GObject *object, GAsyncResult *res, gpointer data)
 static void
 _fetch_groups (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   if (!frogr_controller_is_authorized (self))
     return;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv->groups_fetched = FALSE;
   priv->fetching_groups = TRUE;
 
   _enable_cancellable (self, TRUE);
@@ -1196,14 +1219,15 @@ _fetch_groups_cb (GObject *object, GAsyncResult *res, gpointer data)
 
 static void _fetch_account_info (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   if (!frogr_controller_is_authorized (self))
     return;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv->account_info_fetched = FALSE;
   priv->fetching_account_info = TRUE;
 
   _enable_cancellable (self, FALSE);
@@ -1261,19 +1285,21 @@ _fetch_account_info_cb (GObject *object, GAsyncResult *res, gpointer data)
     }
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
+  priv->account_info_fetched = TRUE;
   priv->fetching_account_info = FALSE;
 }
 
 static void _fetch_account_extra_info (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   if (!frogr_controller_is_authorized (self))
     return;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv->account_extra_info_fetched = FALSE;
   priv->fetching_account_extra_info = TRUE;
 
   _enable_cancellable (self, FALSE);
@@ -1335,20 +1361,27 @@ _fetch_account_extra_info_cb (GObject *object, GAsyncResult *res, gpointer data)
     }
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
+  priv->account_extra_info_fetched = TRUE;
   priv->fetching_account_extra_info = FALSE;
 }
 
 static void
 _fetch_tags (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   if (!frogr_controller_is_authorized (self))
     return;
 
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv->tags_fetched = FALSE;
+
+  /* Do not actually fetch tags if the autocompletion is off */
+  if (!frogr_config_get_tags_autocompletion (priv->config))
+    return;
+
   priv->fetching_tags = TRUE;
 
   _enable_cancellable (self, TRUE);
@@ -1395,6 +1428,47 @@ _fetch_tags_cb (GObject *object, GAsyncResult *res, gpointer data)
   priv->fetching_tags = FALSE;
 }
 
+static gboolean
+_show_progress_on_idle (gpointer data)
+{
+  FrogrController *controller = NULL;
+  FrogrControllerPrivate *priv = NULL;
+  FetchingActivity activity = FETCHING_NOTHING;
+  const gchar *text = NULL;
+  gboolean show_dialog = FALSE;
+
+  controller = frogr_controller_get_instance ();
+  priv = FROGR_CONTROLLER_GET_PRIVATE (controller);
+
+  activity = GPOINTER_TO_INT (data);
+  switch (activity)
+    {
+    case FETCHING_SETS:
+      text = _("Retrieving list of sets…");
+      show_dialog = priv->fetching_tags;
+      break;
+
+    case FETCHING_GROUPS:
+      text = _("Retrieving list of groups…");
+      show_dialog = priv->fetching_groups;
+      break;
+
+    case FETCHING_TAGS:
+      text = _("Retrieving list of tags…");
+      show_dialog = priv->fetching_tags;
+      break;
+
+    default:
+      text = NULL;
+    }
+
+  /* Actually show the dialog, if needed */
+  if (show_dialog)
+    frogr_main_view_show_progress (priv->mainview, text);
+
+  return FALSE;
+}
+
 static gboolean
 _show_details_dialog_on_idle (GSList *pictures)
 {
@@ -1410,7 +1484,7 @@ _show_details_dialog_on_idle (GSList *pictures)
   mainview = priv->mainview;
 
   /* Keep the source while internally busy */
-  if (priv->fetching_tags)
+  if (priv->fetching_tags && frogr_config_get_tags_autocompletion (priv->config))
     {
       frogr_main_view_pulse_progress (mainview);
       return TRUE;
@@ -1442,7 +1516,7 @@ _show_add_tags_dialog_on_idle (GSList *pictures)
   mainview = priv->mainview;
 
   /* Keep the source while internally busy */
-  if (priv->fetching_tags)
+  if (priv->fetching_tags && frogr_config_get_tags_autocompletion (priv->config))
     {
       frogr_main_view_pulse_progress (mainview);
       return TRUE;
@@ -1707,6 +1781,8 @@ frogr_controller_init (FrogrController *self)
   priv->fetching_tags = FALSE;
   priv->adding_to_set = FALSE;
   priv->adding_to_group = FALSE;
+  priv->account_info_fetched = FALSE;
+  priv->account_extra_info_fetched = FALSE;
   priv->sets_fetched = FALSE;
   priv->groups_fetched = FALSE;
   priv->tags_fetched = FALSE;
@@ -1749,19 +1825,22 @@ frogr_controller_get_instance (void)
 FrogrMainView *
 frogr_controller_get_main_view (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FALSE);
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   return priv->mainview;
 }
 
 gboolean
 frogr_controller_run_app (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FALSE);
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   if (priv->app_running)
     {
@@ -1777,7 +1856,7 @@ frogr_controller_run_app (FrogrController *self)
   priv->app_running = TRUE;
 
   /* Try to pre-fetch some data from the server right after launch */
-  _fetch_everything (self);
+  _fetch_everything (self, TRUE);
 
   /* Start on idle state */
   _set_state (self, FROGR_STATE_IDLE);
@@ -1794,9 +1873,11 @@ frogr_controller_run_app (FrogrController *self)
 gboolean
 frogr_controller_quit_app (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FALSE);
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   if (priv->app_running)
     {
@@ -1820,13 +1901,13 @@ void
 frogr_controller_set_active_account (FrogrController *self,
                                      FrogrAccount *account)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
   FrogrAccount *new_account = NULL;
   gboolean accounts_changed = FALSE;
   const gchar *token = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   /* Do nothing if we're setting the same account active again*/
@@ -1836,8 +1917,15 @@ frogr_controller_set_active_account (FrogrController *self,
   new_account = FROGR_IS_ACCOUNT (account) ? g_object_ref (account) : NULL;
   if (new_account)
     {
-      frogr_account_set_is_active (new_account, TRUE);
-      accounts_changed = frogr_config_add_account (priv->config, new_account);
+      const gchar *new_account_id = NULL;
+
+      new_account_id = frogr_account_get_id (new_account);
+      if (!frogr_config_set_active_account (priv->config, new_account_id))
+        {
+          /* Fallback to manually creating a new account */
+          frogr_account_set_is_active (new_account, TRUE);
+          accounts_changed = frogr_config_add_account (priv->config, new_account);
+        }
 
       /* Get the token for setting it later on */
       token = frogr_account_get_token (new_account);
@@ -1857,11 +1945,8 @@ frogr_controller_set_active_account (FrogrController *self,
   fsp_session_set_token (priv->session, token);
 
   /* Prefetch info for this user */
-  priv->sets_fetched = FALSE;
-  priv->groups_fetched = FALSE;
-  priv->tags_fetched = FALSE;
   if (new_account)
-    _fetch_everything (self);
+    _fetch_everything (self, TRUE);
 
   /* Emit proper signals */
   g_signal_emit (self, signals[ACTIVE_ACCOUNT_CHANGED], 0, account);
@@ -1875,27 +1960,33 @@ frogr_controller_set_active_account (FrogrController *self,
 FrogrAccount *
 frogr_controller_get_active_account (FrogrController *self)
 {
-  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UKNOWN);
+  FrogrControllerPrivate *priv = NULL;
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UNKNOWN);
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   return priv->account;
 }
 
 GSList *
 frogr_controller_get_all_accounts (FrogrController *self)
 {
-  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UKNOWN);
+  FrogrControllerPrivate *priv = NULL;
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UNKNOWN);
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   return frogr_config_get_accounts (priv->config);
 }
 
 FrogrControllerState
 frogr_controller_get_state (FrogrController *self)
 {
-  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UKNOWN);
+  FrogrControllerPrivate *priv = NULL;
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FROGR_STATE_UNKNOWN);
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   return priv->state;
 }
 
@@ -1904,10 +1995,10 @@ frogr_controller_set_proxy (FrogrController *self,
                             const char *host, const char *port,
                             const char *username, const char *password)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   /* The host is mandatory to set up a proxy */
@@ -1915,12 +2006,11 @@ frogr_controller_set_proxy (FrogrController *self,
     fsp_session_set_http_proxy (priv->session, NULL, NULL, NULL, NULL);
     DEBUG ("%s", "Not using HTTP proxy");
   } else {
-    gboolean has_port = FALSE;
     gboolean has_username = FALSE;
     gboolean has_password = FALSE;
+    gboolean proxy_changed = FALSE;
     gchar *auth_part = NULL;
 
-    has_port = (port != NULL && *port != '\0');
     has_username = (username != NULL && *username != '\0');
     has_password = (password != NULL && *password != '\0');
 
@@ -1930,17 +2020,38 @@ frogr_controller_set_proxy (FrogrController *self,
     DEBUG ("Using HTTP proxy: %s%s:%s", auth_part ? auth_part : "", host, port);
     g_free (auth_part);
 
-    fsp_session_set_http_proxy (priv->session, host, port, username, password);
+    proxy_changed = fsp_session_set_http_proxy (priv->session,
+                                                host, port,
+                                                username, password);
+
+    /* Re-fetch information if needed after changing proxy configuration */
+    if (priv->app_running && proxy_changed)
+      _fetch_everything (self, FALSE);
   }
 }
 
+void
+frogr_controller_fetch_tags_if_needed (FrogrController *self)
+{
+  FrogrControllerPrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  if (!priv->fetching_tags && !priv->tags_fetched)
+    _fetch_tags (self);
+}
+
 void
 frogr_controller_show_about_dialog (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+  GtkWindow *window = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  GtkWindow *window = frogr_main_view_get_window (priv->mainview);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  window = frogr_main_view_get_window (priv->mainview);
 
   /* Run the about dialog */
   frogr_about_dialog_show (window);
@@ -1949,10 +2060,13 @@ frogr_controller_show_about_dialog (FrogrController *self)
 void
 frogr_controller_show_auth_dialog (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+  GtkWindow *window = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  GtkWindow *window = frogr_main_view_get_window (priv->mainview);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  window = frogr_main_view_get_window (priv->mainview);
 
   /* Run the auth dialog */
   frogr_auth_dialog_show (window, REQUEST_AUTHORIZATION);
@@ -1961,10 +2075,13 @@ frogr_controller_show_auth_dialog (FrogrController *self)
 void
 frogr_controller_show_settings_dialog (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+  GtkWindow *window = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  GtkWindow *window = frogr_main_view_get_window (priv->mainview);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  window = frogr_main_view_get_window (priv->mainview);
 
   /* Run the auth dialog */
   frogr_settings_dialog_show (window);
@@ -1974,21 +2091,18 @@ void
 frogr_controller_show_details_dialog (FrogrController *self,
                                       GSList *pictures)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
-  GSList *tags_list = NULL;
-
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  mainview_model = frogr_main_view_get_model (priv->mainview);
-  tags_list = frogr_main_view_model_get_tags_list (mainview_model);
 
   /* Fetch the tags list first if needed */
-  if (frogr_main_view_model_n_tags (mainview_model) == 0 && !priv->tags_fetched)
+  if (frogr_config_get_tags_autocompletion (priv->config) && !priv->tags_fetched)
     {
-      frogr_main_view_show_progress (priv->mainview, _("Retrieving list of tags…"));
-      _fetch_tags (self);
+      gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_TAGS));
+      if (!priv->fetching_tags)
+        _fetch_tags (self);
     }
 
   /* Show the dialog when possible */
@@ -1999,19 +2113,18 @@ void
 frogr_controller_show_add_tags_dialog (FrogrController *self,
                                        GSList *pictures)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
-
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  mainview_model = frogr_main_view_get_model (priv->mainview);
 
   /* Fetch the tags list first if needed */
-  if (frogr_main_view_model_n_tags (mainview_model) == 0 && !priv->tags_fetched)
+  if (frogr_config_get_tags_autocompletion (priv->config) && !priv->tags_fetched)
     {
-      frogr_main_view_show_progress (priv->mainview, _("Retrieving list of tags…"));
-      _fetch_tags (self);
+      gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_TAGS));
+      if (!priv->fetching_tags)
+        _fetch_tags (self);
     }
 
   /* Show the dialog when possible */
@@ -2022,19 +2135,18 @@ void
 frogr_controller_show_create_new_set_dialog (FrogrController *self,
                                              GSList *pictures)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
-
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  mainview_model = frogr_main_view_get_model (priv->mainview);
 
   /* Fetch the sets first if needed */
-  if (frogr_main_view_model_n_sets (mainview_model) == 0 && !priv->sets_fetched)
+  if (!priv->sets_fetched)
     {
-      frogr_main_view_show_progress (priv->mainview, _("Retrieving list of sets…"));
-      _fetch_sets (self);
+      gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_SETS));
+      if (!priv->fetching_sets)
+        _fetch_sets (self);
     }
 
   /* Show the dialog when possible */
@@ -2045,19 +2157,18 @@ void
 frogr_controller_show_add_to_set_dialog (FrogrController *self,
                                          GSList *pictures)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
-
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  mainview_model = frogr_main_view_get_model (priv->mainview);
 
   /* Fetch the sets first if needed */
-  if (frogr_main_view_model_n_sets (mainview_model) == 0 && !priv->sets_fetched)
+  if (!priv->sets_fetched)
     {
-      frogr_main_view_show_progress (priv->mainview, _("Retrieving list of sets…"));
-      _fetch_sets (self);
+      gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_SETS));
+      if (!priv->fetching_sets)
+        _fetch_sets (self);
     }
 
   /* Show the dialog when possible */
@@ -2068,19 +2179,18 @@ void
 frogr_controller_show_add_to_group_dialog (FrogrController *self,
                                            GSList *pictures)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = NULL;
-  FrogrMainViewModel *mainview_model = NULL;
-
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
-  mainview_model = frogr_main_view_get_model (priv->mainview);
 
   /* Fetch the groups first if needed */
-  if (frogr_main_view_model_n_groups (mainview_model) == 0 && !priv->groups_fetched)
+  if (!priv->groups_fetched)
     {
-      frogr_main_view_show_progress (priv->mainview, _("Retrieving list of groups…"));
-      _fetch_groups (self);
+      gdk_threads_add_timeout (DEFAULT_TIMEOUT, (GSourceFunc) _show_progress_on_idle, GINT_TO_POINTER (FETCHING_GROUPS));
+      if (!priv->fetching_groups)
+        _fetch_groups (self);
     }
 
   /* Show the dialog when possible */
@@ -2090,10 +2200,10 @@ frogr_controller_show_add_to_group_dialog (FrogrController *self,
 void
 frogr_controller_open_auth_url (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   _enable_cancellable (self, FALSE);
   fsp_session_get_auth_url_async (priv->session, NULL, _get_auth_url_cb, self);
 }
@@ -2101,10 +2211,10 @@ frogr_controller_open_auth_url (FrogrController *self)
 void
 frogr_controller_complete_auth (FrogrController *self)
 {
-  g_return_if_fail(FROGR_IS_CONTROLLER (self));
-
   FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   _enable_cancellable (self, FALSE);
   fsp_session_complete_auth_async (priv->session, NULL, _complete_auth_cb, self);
 }
@@ -2112,18 +2222,22 @@ frogr_controller_complete_auth (FrogrController *self)
 gboolean
 frogr_controller_is_authorized (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_CONTROLLER (self), FALSE);
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   return (priv->account != NULL);
 }
 
 void
 frogr_controller_revoke_authorization (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   /* Ensure there's the token/account is no longer active anywhere */
   fsp_session_set_token (priv->session, NULL);
@@ -2132,15 +2246,15 @@ frogr_controller_revoke_authorization (FrogrController *self)
 
 void
 frogr_controller_load_pictures (FrogrController *self,
-                                GSList *filepaths)
+                                GSList *fileuris)
 {
   FrogrPictureLoader *fploader;
-  fploader = frogr_picture_loader_new (filepaths,
+  fploader = frogr_picture_loader_new (fileuris,
                                        (FrogrPictureLoadedCallback) _on_picture_loaded,
                                        (FrogrPicturesLoadedCallback) _on_pictures_loaded,
                                        self);
   /* Load the pictures! */
-  _set_state (self, FROGR_STATE_BUSY);
+  _set_state (self, FROGR_STATE_LOADING_PICTURES);
 
   frogr_picture_loader_load (fploader);
 }
@@ -2148,9 +2262,11 @@ frogr_controller_load_pictures (FrogrController *self,
 void
 frogr_controller_upload_pictures (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
-  FrogrControllerPrivate *priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
 
   /* Upload pictures */
   if (!frogr_controller_is_authorized (self))
@@ -2181,19 +2297,30 @@ frogr_controller_upload_pictures (FrogrController *self)
                                         (FrogrPicturesUploadedCallback) _on_pictures_uploaded,
                                         self);
           /* Load the pictures! */
-          _set_state (self, FROGR_STATE_BUSY);
+          _set_state (self, FROGR_STATE_UPLOADING_PICTURES);
           frogr_picture_uploader_upload (fpuploader);
         }
     }
 }
 
 void
-frogr_controller_cancel_ongoing_request (FrogrController *self)
+frogr_controller_reorder_pictures (FrogrController *self)
 {
+  FrogrControllerPrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_CONTROLLER (self));
 
+  priv = FROGR_CONTROLLER_GET_PRIVATE (self);
+  frogr_main_view_reorder_pictures (priv->mainview);
+}
+
+void
+frogr_controller_cancel_ongoing_request (FrogrController *self)
+{
   FrogrControllerPrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_CONTROLLER (self));
+
   priv = FROGR_CONTROLLER_GET_PRIVATE (self);
   if (!G_IS_CANCELLABLE (priv->cancellable)
       || g_cancellable_is_cancelled (priv->cancellable))
