@@ -1,7 +1,7 @@
 /*
  * frogr-controller.h -- Controller of the whole application
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -53,11 +53,14 @@ struct _FrogrController
 };
 
 typedef enum {
-  FROGR_STATE_UKNOWN,
+  FROGR_STATE_UNKNOWN,
   FROGR_STATE_IDLE,
-  FROGR_STATE_BUSY
+  FROGR_STATE_LOADING_PICTURES,
+  FROGR_STATE_UPLOADING_PICTURES
 } FrogrControllerState;
 
+#define FROGR_STATE_IS_BUSY(state) ((state) != FROGR_STATE_UNKNOWN && (state) != FROGR_STATE_IDLE)
+
 GType frogr_controller_get_type (void) G_GNUC_CONST;
 
 FrogrController *frogr_controller_get_instance (void);
@@ -81,6 +84,8 @@ void frogr_controller_set_proxy (FrogrController *self,
                                  const char *host, const char *port,
                                  const char *username, const char *password);
 
+void frogr_controller_fetch_tags_if_needed (FrogrController *self);
+
 void frogr_controller_show_about_dialog (FrogrController *self);
 
 void frogr_controller_show_auth_dialog (FrogrController *self);
@@ -110,10 +115,12 @@ gboolean frogr_controller_is_authorized (FrogrController *self);
 
 void frogr_controller_revoke_authorization (FrogrController *self);
 
-void frogr_controller_load_pictures (FrogrController *self, GSList *filepaths);
+void frogr_controller_load_pictures (FrogrController *self, GSList *fileuris);
 
 void frogr_controller_upload_pictures (FrogrController *self);
 
+void frogr_controller_reorder_pictures (FrogrController *self);
+
 void frogr_controller_cancel_ongoing_request (FrogrController *self);
 
 G_END_DECLS
