@@ -1,7 +1,7 @@
 /*
  * frogr-global-defs.h -- Global definitions and constants
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -25,7 +25,7 @@
 
 #include <config.h>
 
-#define APP_NAME "Flickr Remote Organizer"
+#define APP_NAME N_("Flickr Remote Organizer")
 #define APP_SHORTNAME PACKAGE
 #define APP_VERSION VERSION
 
