@@ -1,7 +1,7 @@
 /*
  * frogr-picture-loader.c -- Asynchronous picture loader in frogr
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,6 +29,12 @@
 #include "frogr-picture.h"
 
 #include <config.h>
+#include <libexif/exif-byte-order.h>
+#include <libexif/exif-data.h>
+#include <libexif/exif-entry.h>
+#include <libexif/exif-format.h>
+#include <libexif/exif-loader.h>
+#include <libexif/exif-tag.h>
 #include <glib/gi18n.h>
 #include <gio/gio.h>
 
@@ -40,7 +46,7 @@
                                 FROGR_TYPE_PICTURE_LOADER,      \
                                 FrogrPictureLoaderPrivate))
 
-G_DEFINE_TYPE (FrogrPictureLoader, frogr_picture_loader, G_TYPE_OBJECT);
+G_DEFINE_TYPE (FrogrPictureLoader, frogr_picture_loader, G_TYPE_OBJECT)
 
 /* Private struct */
 typedef struct _FrogrPictureLoaderPrivate FrogrPictureLoaderPrivate;
@@ -48,18 +54,28 @@ struct _FrogrPictureLoaderPrivate
 {
   FrogrController *controller;
   FrogrMainView *mainview;
-  FrogrConfig *config;
 
-  GSList *filepaths;
+  GSList *file_uris;
   GSList *current;
   guint index;
   guint n_pictures;
 
+  gboolean keep_file_extensions;
+  gboolean public_visibility;
+  gboolean family_visibility;
+  gboolean friend_visibility;
+  gboolean show_in_search;
+  FspSafetyLevel safety_level;
+  FspContentType content_type;
+
   FrogrPictureLoadedCallback picture_loaded_cb;
   FrogrPicturesLoadedCallback pictures_loaded_cb;
   GObject *object;
 };
 
+#ifndef MAC_INTEGRATION
+/* Don't use this in Mac OSX, where GNOME VFS daemon won't be running,
+   so we couldn't reliably check mime types (will be text/plain) */
 static const gchar *valid_mimetypes[] = {
   "image/jpg",
   "image/jpeg",
@@ -67,11 +83,12 @@ static const gchar *valid_mimetypes[] = {
   "image/bmp",
   "image/gif",
   NULL};
+#endif
 
 /* Prototypes */
 
 static void _update_status_and_progress (FrogrPictureLoader *self);
-static GdkPixbuf *_get_scaled_pixbuf (GdkPixbuf *pixbuf);
+static GdkPixbuf *_get_corrected_pixbuf (GdkPixbuf *pixbuf);
 static void _load_next_picture (FrogrPictureLoader *self);
 static void _load_next_picture_cb (GObject *object,
                                    GAsyncResult *res,
@@ -99,9 +116,11 @@ _update_status_and_progress (FrogrPictureLoader *self)
 }
 
 static GdkPixbuf *
-_get_scaled_pixbuf (GdkPixbuf *pixbuf)
+_get_corrected_pixbuf (GdkPixbuf *pixbuf)
 {
-  GdkPixbuf *scaled_pixbuf = NULL;
+  GdkPixbuf *scaled_pixbuf;
+  GdkPixbuf *rotated_pixbuf;
+  const gchar *orientation;
   gint width;
   gint height;
   gint new_width;
@@ -125,6 +144,35 @@ _get_scaled_pixbuf (GdkPixbuf *pixbuf)
   scaled_pixbuf = gdk_pixbuf_scale_simple (pixbuf,
                                            new_width, new_height,
                                            GDK_INTERP_TILES);
+  /* Correct orientation if needed */
+  orientation = gdk_pixbuf_get_option (pixbuf, "orientation");
+
+  /* No orientation defined or 0 degrees rotation: we're done */
+  if (!orientation || !g_strcmp0 (orientation, "1"))
+    return scaled_pixbuf;
+
+  DEBUG ("File orientation for file: %s", orientation);
+  rotated_pixbuf = NULL;
+
+  /* Rotated 90 degrees */
+  if (!g_strcmp0 (orientation, "8"))
+    rotated_pixbuf = gdk_pixbuf_rotate_simple (scaled_pixbuf,
+                                               GDK_PIXBUF_ROTATE_COUNTERCLOCKWISE);
+  /* Rotated 180 degrees */
+  if (!g_strcmp0 (orientation, "3"))
+    rotated_pixbuf = gdk_pixbuf_rotate_simple (scaled_pixbuf,
+                                               GDK_PIXBUF_ROTATE_UPSIDEDOWN);
+  /* Rotated 270 degrees */
+  if (!g_strcmp0 (orientation, "6"))
+    rotated_pixbuf = gdk_pixbuf_rotate_simple (scaled_pixbuf,
+                                               GDK_PIXBUF_ROTATE_CLOCKWISE);
+  if (rotated_pixbuf)
+    {
+      g_object_unref (scaled_pixbuf);
+      return rotated_pixbuf;
+    }
+
+  /* No rotation was applied, return the scaled pixbuf */
   return scaled_pixbuf;
 }
 
@@ -136,22 +184,26 @@ _load_next_picture (FrogrPictureLoader *self)
 
   if (priv->current)
     {
-      GFile *gfile = NULL;
+      gchar *file_uri = (gchar *)priv->current->data;
+      GFile *gfile = g_file_new_for_uri (file_uri);
+      gboolean valid_mime = TRUE;
+
+#ifndef MAC_INTEGRATION
       GFileInfo *file_info;
-      gchar *filepath = (gchar *)priv->current->data;
       const gchar *mime_type;
-      gboolean valid_mime = FALSE;
       gint i;
 
       /* Get file info */
-      gfile = g_file_new_for_path (filepath);
       file_info = g_file_query_info (gfile,
                                      G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
                                      G_FILE_QUERY_INFO_NONE,
                                      NULL,
                                      NULL);
-      /* Check mimetype */
+
+      /* Check mimetype (only when not in Mac OSX) */
       mime_type = g_file_info_get_content_type (file_info);
+
+      valid_mime = FALSE;
       for (i = 0; valid_mimetypes[i]; i++)
         {
           if (g_str_equal (valid_mimetypes[i], mime_type))
@@ -161,8 +213,11 @@ _load_next_picture (FrogrPictureLoader *self)
             }
         }
 
-      DEBUG ("Adding file %s (%s)", filepath, mime_type);
+      DEBUG ("Mime detected: %s)", mime_type);
       g_object_unref (file_info);
+#endif
+
+      DEBUG ("Adding file %s", file_uri);
 
       /* Asynchronously load the picture if mime is valid */
       if (valid_mime)
@@ -199,16 +254,18 @@ _load_next_picture_cb (GObject *object,
                        GAsyncResult *res,
                        gpointer data)
 {
-  FrogrPictureLoader *self = FROGR_PICTURE_LOADER (data);;
-  FrogrPictureLoaderPrivate *priv =
-    FROGR_PICTURE_LOADER_GET_PRIVATE (self);
-
+  FrogrPictureLoader *self = NULL;
+  FrogrPictureLoaderPrivate *priv = NULL;
   FrogrPicture *fpicture = NULL;
-  GFile *file = G_FILE (object);
+  GFile *file = NULL;
   GError *error = NULL;
-  gchar *contents;
-  gsize length;
+  gchar *contents = NULL;
+  gsize length = 0;
 
+  self = FROGR_PICTURE_LOADER (data);;
+  priv = FROGR_PICTURE_LOADER_GET_PRIVATE (self);
+
+  file = G_FILE (object);
   if (g_file_load_contents_finish (file, res, &contents, &length, NULL, &error))
     {
       GdkPixbufLoader *pixbuf_loader = gdk_pixbuf_loader_new ();
@@ -218,39 +275,100 @@ _load_next_picture_cb (GObject *object,
                                    length,
                                    &error))
         {
-          GdkPixbuf *pixbuf;
-          GdkPixbuf *s_pixbuf;
-          gchar *filepath;
-          gchar *filename;
+          GdkPixbuf *pixbuf = NULL;
+          GdkPixbuf *s_pixbuf = NULL;
+          GFileInfo* file_info = NULL;
+          ExifLoader *exif_loader = NULL;
+          ExifData *exif_data = NULL;
+          ExifEntry *exif_entry = NULL;
+          gchar *file_uri = NULL;
+          gchar *file_name = NULL;
+          guint64 filesize = 0;
 
           /* Gather needed information */
-          filepath = g_file_get_path (file);
-          filename = g_file_get_basename (file);
+          file_info = g_file_query_info (file,
+                                         G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME
+                                         "," G_FILE_ATTRIBUTE_STANDARD_SIZE,
+                                         G_FILE_QUERY_INFO_NONE,
+                                         NULL, &error);
+          if (!error)
+            {
+              file_name = g_strdup (g_file_info_get_display_name (file_info));
+              filesize = g_file_info_get_size (file_info);
+            }
+          else
+            {
+              g_warning ("Not able to write pixbuf: %s", error->message);
+              g_error_free (error);
+
+              /* Fallback if g_file_query_info() failed */
+              file_name = g_file_get_basename (file);
+            }
+
+          if (!priv->keep_file_extensions)
+            {
+              gchar *extension_dot = NULL;
+
+              /* Remove extension if present */
+              extension_dot = g_strrstr (file_name, ".");
+              if (extension_dot)
+                *extension_dot = '\0';
+            }
+
+          file_uri = g_file_get_uri (file);
           gdk_pixbuf_loader_close (pixbuf_loader, NULL);
           pixbuf = gdk_pixbuf_loader_get_pixbuf (pixbuf_loader);
 
-          /* Get (scaled) pixbuf */
-          s_pixbuf = _get_scaled_pixbuf (pixbuf);
+          /* Get (scaled, and maybe rotated) pixbuf */
+          s_pixbuf = _get_corrected_pixbuf (pixbuf);
 
-          /* Build the FrogrPicture and set pixbuf */
-          fpicture = frogr_picture_new (filepath,
-                                        filename,
-                                        frogr_config_get_default_public (priv->config),
-                                        frogr_config_get_default_family (priv->config),
-                                        frogr_config_get_default_friend (priv->config));
-          frogr_picture_set_show_in_search (fpicture,
-                                            frogr_config_get_default_show_in_search (priv->config));
-          frogr_picture_set_content_type (fpicture,
-                                          frogr_config_get_default_content_type (priv->config));
-          frogr_picture_set_safety_level (fpicture,
-                                          frogr_config_get_default_safety_level (priv->config));
+          /* Build the FrogrPicture */
+          fpicture = frogr_picture_new (file_uri,
+                                        file_name,
+                                        priv->public_visibility,
+                                        priv->family_visibility,
+                                        priv->friend_visibility);
 
+          frogr_picture_set_show_in_search (fpicture, priv->show_in_search);
+          frogr_picture_set_content_type (fpicture, priv->content_type);
+          frogr_picture_set_safety_level (fpicture, priv->safety_level);
           frogr_picture_set_pixbuf (fpicture, s_pixbuf);
 
-          /* Free */
+          /* FrogrPicture stores the size in KB */
+          frogr_picture_set_filesize (fpicture, filesize / 1024);
+
+          /* Set date and time from exif data, if present */
+          exif_loader = exif_loader_new();
+          exif_loader_write (exif_loader, (unsigned char *) contents, length);
+
+          exif_data = exif_loader_get_data (exif_loader);
+          if (exif_data)
+            {
+              exif_entry = exif_data_get_entry (exif_data, EXIF_TAG_DATE_TIME);
+              if (exif_entry)
+                {
+                  if (exif_entry->format == EXIF_FORMAT_ASCII)
+                    {
+                      gchar *value = g_new0 (char, 20);
+                      exif_entry_get_value (exif_entry, value, 20);
+
+                      frogr_picture_set_datetime (fpicture, value);
+                      g_free (value);
+                    }
+                  else
+                    g_warning ("Found DateTime exif tag of invalid type");
+                }
+              exif_data_unref (exif_data);
+            }
+
+          if (file_info)
+            g_object_unref (file_info);
+
+          exif_loader_unref (exif_loader);
+
           g_object_unref (s_pixbuf);
-          g_free (filepath);
-          g_free (filename);
+          g_free (file_uri);
+          g_free (file_name);
         }
       else
         {
@@ -265,12 +383,12 @@ _load_next_picture_cb (GObject *object,
   else
     {
       /* Not able to load contents */
-      gchar *filename = g_file_get_basename (file);
+      gchar *file_name = g_file_get_basename (file);
       g_warning ("Not able to read contents from %s: %s",
-                 filename,
+                 file_name,
                  error->message);
       g_error_free (error);
-      g_free (filename);
+      g_free (file_name);
     }
 
   g_object_unref (file);
@@ -313,12 +431,6 @@ _frogr_picture_loader_dispose (GObject* object)
       priv->controller = NULL;
     }
 
-  if (priv->config)
-    {
-      g_object_unref (priv->config);
-      priv->config = NULL;
-    }
-
   G_OBJECT_CLASS (frogr_picture_loader_parent_class)->dispose(object);
 }
 
@@ -329,8 +441,8 @@ _frogr_picture_loader_finalize (GObject* object)
     FROGR_PICTURE_LOADER_GET_PRIVATE (object);
 
   /* Free */
-  g_slist_foreach (priv->filepaths, (GFunc)g_free, NULL);
-  g_slist_free (priv->filepaths);
+  g_slist_foreach (priv->file_uris, (GFunc)g_free, NULL);
+  g_slist_free (priv->file_uris);
 
   G_OBJECT_CLASS (frogr_picture_loader_parent_class)->finalize(object);
 }
@@ -352,15 +464,25 @@ frogr_picture_loader_init (FrogrPictureLoader *self)
   FrogrPictureLoaderPrivate *priv =
     FROGR_PICTURE_LOADER_GET_PRIVATE (self);
 
+  FrogrConfig *config = frogr_config_get_instance ();
+
   /* Init private data */
 
   /* We need the controller to get the main window */
   priv->controller = g_object_ref (frogr_controller_get_instance ());
   priv->mainview = g_object_ref (frogr_controller_get_main_view (priv->controller));
-  priv->config = g_object_ref (frogr_config_get_instance ());
+
+  /* Initialize values from frogr configuration */
+  priv->keep_file_extensions = frogr_config_get_keep_file_extensions (config);
+  priv->public_visibility = frogr_config_get_default_public (config);
+  priv->family_visibility = frogr_config_get_default_family (config);
+  priv->friend_visibility = frogr_config_get_default_friend (config);
+  priv->show_in_search = frogr_config_get_default_show_in_search (config);
+  priv->safety_level = frogr_config_get_default_safety_level (config);
+  priv->content_type = frogr_config_get_default_content_type (config);
 
   /* Init the rest of private data */
-  priv->filepaths = NULL;
+  priv->file_uris = NULL;
   priv->current = NULL;
   priv->index = -1;
   priv->n_pictures = 0;
@@ -369,7 +491,7 @@ frogr_picture_loader_init (FrogrPictureLoader *self)
 /* Public API */
 
 FrogrPictureLoader *
-frogr_picture_loader_new (GSList *filepaths,
+frogr_picture_loader_new (GSList *file_uris,
                           FrogrPictureLoadedCallback picture_loaded_cb,
                           FrogrPicturesLoadedCallback pictures_loaded_cb,
                           gpointer object)
@@ -381,10 +503,10 @@ frogr_picture_loader_new (GSList *filepaths,
     FROGR_PICTURE_LOADER_GET_PRIVATE (self);
 
   /* Internal data */
-  priv->filepaths = g_slist_copy (filepaths);
-  priv->current = priv->filepaths;
+  priv->file_uris = g_slist_copy (file_uris);
+  priv->current = priv->file_uris;
   priv->index = 0;
-  priv->n_pictures = g_slist_length (priv->filepaths);
+  priv->n_pictures = g_slist_length (priv->file_uris);
 
   /* Callback data */
   priv->picture_loaded_cb = picture_loaded_cb;
@@ -397,13 +519,14 @@ frogr_picture_loader_new (GSList *filepaths,
 void
 frogr_picture_loader_load (FrogrPictureLoader *self)
 {
+  FrogrPictureLoaderPrivate *priv = NULL;
+
   g_return_if_fail (FROGR_IS_PICTURE_LOADER (self));
 
-  FrogrPictureLoaderPrivate *priv =
-    FROGR_PICTURE_LOADER_GET_PRIVATE (self);
+  priv = FROGR_PICTURE_LOADER_GET_PRIVATE (self);
 
   /* Check first whether there's something to load */
-  if (priv->filepaths == NULL)
+  if (priv->file_uris == NULL)
     return;
 
   /* Update status and progress */
