@@ -1,7 +1,7 @@
 /*
  * frogr-picture-loader.h -- Asynchronous picture loader in frogr
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -60,7 +60,7 @@ struct _FrogrPictureLoaderClass
 
 GType frogr_picture_loader_get_type(void) G_GNUC_CONST;
 
-FrogrPictureLoader *frogr_picture_loader_new (GSList *filepaths,
+FrogrPictureLoader *frogr_picture_loader_new (GSList *file_uris,
                                               FrogrPictureLoadedCallback picture_loaded_cb,
                                               FrogrPicturesLoadedCallback pictures_loaded_cb,
                                               gpointer object);
