@@ -1,7 +1,7 @@
 /*
  * frogr-picture-uploader.c -- Asynchronous picture uploader in frogr
  *
- * Copyright (C) 2009, 2010 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -36,7 +36,7 @@
                                 FROGR_TYPE_PICTURE_UPLOADER,    \
                                 FrogrPictureUploaderPrivate))
 
-G_DEFINE_TYPE (FrogrPictureUploader, frogr_picture_uploader, G_TYPE_OBJECT);
+G_DEFINE_TYPE (FrogrPictureUploader, frogr_picture_uploader, G_TYPE_OBJECT)
 
 /* Private struct */
 typedef struct _FrogrPictureUploaderPrivate FrogrPictureUploaderPrivate;
@@ -73,8 +73,8 @@ _update_status_and_progress (FrogrPictureUploader *self)
   FrogrPictureUploaderPrivate *priv =
     FROGR_PICTURE_UPLOADER_GET_PRIVATE (self);
 
+  gchar *description = NULL;
   gchar *status_text = NULL;
-  gchar *progress_bar_text = NULL;
 
   if (priv->current)
     {
@@ -82,20 +82,18 @@ _update_status_and_progress (FrogrPictureUploader *self)
       gchar *title = g_strdup (frogr_picture_get_title (picture));
 
       /* Update progress */
-      status_text = g_strdup_printf (_("Uploading '%s'..."), title);
-      progress_bar_text = g_strdup_printf ("%d / %d",
-                                           priv->index,
+      description = g_strdup_printf (_("Uploading '%s'…"), title);
+      status_text = g_strdup_printf ("%d / %d",
+                                           priv->index + 1,
                                            priv->n_pictures);
       g_free (title);
     }
+  frogr_main_view_set_progress_description(priv->mainview, description);
+  frogr_main_view_set_progress_status_text (priv->mainview, status_text);
 
-  frogr_main_view_set_status_text (priv->mainview, status_text);
-  frogr_main_view_set_progress (priv->mainview,
-                                (double) priv->index / priv->n_pictures,
-                                progress_bar_text);
   /* Free */
+  g_free (description);
   g_free (status_text);
-  g_free (progress_bar_text);
 }
 
 static void
@@ -116,12 +114,15 @@ _upload_next_picture (FrogrPictureUploader *self)
     }
   else
     {
-      /* Update status and progress bars */
-      _update_status_and_progress (self);
+      /* Hide progress bar dialog */
+      frogr_main_view_hide_progress (priv->mainview);
 
       /* Execute final callback */
       if (priv->pictures_uploaded_cb)
-        priv->pictures_uploaded_cb (priv->object, self, priv->error);
+        priv->pictures_uploaded_cb (priv->object, priv->error);
+
+      /* Process finished, self-destruct */
+      g_object_unref (self);
     }
 }
 
@@ -245,16 +246,18 @@ frogr_picture_uploader_new (GSList *pictures,
 void
 frogr_picture_uploader_upload (FrogrPictureUploader *self)
 {
+  FrogrPictureUploaderPrivate *priv = NULL;
+
   g_return_if_fail (FROGR_IS_PICTURE_UPLOADER (self));
 
-  FrogrPictureUploaderPrivate *priv =
-    FROGR_PICTURE_UPLOADER_GET_PRIVATE (self);
+  priv = FROGR_PICTURE_UPLOADER_GET_PRIVATE (self);
 
   /* Check first whether there's something to upload */
   if (priv->pictures == NULL)
     return;
 
-  /* Update status and progress bars */
+  /* Update status and show progress bars */
+  frogr_main_view_show_progress (priv->mainview, NULL);
   _update_status_and_progress (self);
 
   /* Trigger the asynchronous process */
