@@ -1,7 +1,7 @@
 /*
  * frogr-picture.c -- A picture in frogr
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -29,19 +29,22 @@
                                 FROGR_TYPE_PICTURE,     \
                                 FrogrPicturePrivate))
 
-G_DEFINE_TYPE (FrogrPicture, frogr_picture, G_TYPE_OBJECT);
+G_DEFINE_TYPE (FrogrPicture, frogr_picture, G_TYPE_OBJECT)
 
 /* Private struct */
 typedef struct _FrogrPicturePrivate FrogrPicturePrivate;
 struct _FrogrPicturePrivate
 {
   gchar *id;
-  gchar *filepath;
+  gchar *fileuri;
   gchar *title;
   gchar *description;
   gchar *tags_string;
   GSList *tags_list;
 
+  gulong filesize; /* In KB */
+  gchar *datetime; /* ASCII, locale dependent, string */
+
   GSList *sets;
   GSList *groups;
 
@@ -60,7 +63,7 @@ struct _FrogrPicturePrivate
 enum  {
   PROP_0,
   PROP_ID,
-  PROP_FILEPATH,
+  PROP_FILEURI,
   PROP_TITLE,
   PROP_DESCRIPTION,
   PROP_TAGS_STRING,
@@ -70,7 +73,9 @@ enum  {
   PROP_SAFETY_LEVEL,
   PROP_CONTENT_TYPE,
   PROP_SHOW_IN_SEARCH,
-  PROP_PIXBUF
+  PROP_PIXBUF,
+  PROP_FILESIZE,
+  PROP_DATETIME
 };
 
 /* Prototypes */
@@ -186,8 +191,8 @@ _frogr_picture_set_property (GObject *object,
     case PROP_ID:
       frogr_picture_set_id (self, g_value_get_string (value));
       break;
-    case PROP_FILEPATH:
-      frogr_picture_set_filepath (self, g_value_get_string (value));
+    case PROP_FILEURI:
+      frogr_picture_set_fileuri (self, g_value_get_string (value));
       break;
     case PROP_TITLE:
       frogr_picture_set_title (self, g_value_get_string (value));
@@ -219,6 +224,12 @@ _frogr_picture_set_property (GObject *object,
     case PROP_PIXBUF:
       frogr_picture_set_pixbuf (self, GDK_PIXBUF (g_value_get_object (value)));
       break;
+    case PROP_FILESIZE:
+      frogr_picture_set_filesize (self, g_value_get_long (value));
+      break;
+    case PROP_DATETIME:
+      frogr_picture_set_datetime (self, g_value_get_string (value));
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -238,8 +249,8 @@ _frogr_picture_get_property (GObject *object,
     case PROP_ID:
       g_value_set_string (value, priv->id);
       break;
-    case PROP_FILEPATH:
-      g_value_set_string (value, priv->filepath);
+    case PROP_FILEURI:
+      g_value_set_string (value, priv->fileuri);
       break;
     case PROP_TITLE:
       g_value_set_string (value, priv->title);
@@ -271,6 +282,12 @@ _frogr_picture_get_property (GObject *object,
     case PROP_PIXBUF:
       g_value_set_object (value, priv->pixbuf);
       break;
+    case PROP_FILESIZE:
+      g_value_set_long (value, priv->filesize);
+      break;
+    case PROP_DATETIME:
+      g_value_set_string (value, priv->datetime);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -314,10 +331,11 @@ _frogr_picture_finalize (GObject* object)
 
   /* free strings */
   g_free (priv->id);
-  g_free (priv->filepath);
+  g_free (priv->fileuri);
   g_free (priv->title);
   g_free (priv->description);
   g_free (priv->tags_string);
+  g_free (priv->datetime);
 
   /* free GSList of tags */
   g_slist_foreach (priv->tags_list, (GFunc) g_free, NULL);
@@ -348,10 +366,10 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                         G_PARAM_READWRITE));
 
   g_object_class_install_property (obj_class,
-                                   PROP_FILEPATH,
-                                   g_param_spec_string ("filepath",
-                                                        "filepath",
-                                                        "Full filepath at disk "
+                                   PROP_FILEURI,
+                                   g_param_spec_string ("fileuri",
+                                                        "fileuri",
+                                                        "Full fileuri at disk "
                                                         "for the picture",
                                                         NULL,
                                                         G_PARAM_READWRITE));
@@ -440,6 +458,22 @@ frogr_picture_class_init(FrogrPictureClass *klass)
                                                         "for this picture",
                                                         GDK_TYPE_PIXBUF,
                                                         G_PARAM_READWRITE));
+  g_object_class_install_property (obj_class,
+                                   PROP_FILESIZE,
+                                   g_param_spec_long ("filesize",
+                                                      "filesize",
+                                                      "Filesize in KB for the file",
+                                                      G_MINLONG,
+                                                      G_MAXLONG,
+                                                      0,
+                                                      G_PARAM_READWRITE));
+  g_object_class_install_property (obj_class,
+                                   PROP_DATETIME,
+                                   g_param_spec_string ("datetime",
+                                                        "datetime",
+                                                        "Date and time string for the file",
+                                                        NULL,
+                                                        G_PARAM_READWRITE));
 
   g_type_class_add_private (obj_class, sizeof (FrogrPicturePrivate));
 }
@@ -451,11 +485,14 @@ frogr_picture_init (FrogrPicture *self)
 
   /* Default values */
   priv->id = NULL;
-  priv->filepath = NULL;
+  priv->fileuri = NULL;
   priv->title = NULL;
   priv->description = NULL;
   priv->tags_string = NULL;
 
+  priv->filesize = 0;
+  priv->datetime = NULL;
+
   priv->tags_list = NULL;
   priv->sets = NULL;
   priv->groups = NULL;
@@ -475,26 +512,25 @@ frogr_picture_init (FrogrPicture *self)
 /* Public API */
 
 FrogrPicture *
-frogr_picture_new (const gchar *filepath,
+frogr_picture_new (const gchar *fileuri,
                    const gchar *title,
                    gboolean public,
                    gboolean family,
                    gboolean friend)
 {
-  g_return_val_if_fail (filepath, NULL);
+  g_return_val_if_fail (fileuri, NULL);
   g_return_val_if_fail (title, NULL);
 
-  GObject *new = g_object_new(FROGR_TYPE_PICTURE,
-                              "filepath", filepath,
-                              "title", title,
-                              "is-public", public,
-                              "is-family", family,
-                              "is-friend", friend,
-                              "safety-level", FSP_SAFETY_LEVEL_SAFE,
-                              "content-type", FSP_CONTENT_TYPE_PHOTO,
-                              "show_in_search", TRUE,
-                              NULL);
-  return FROGR_PICTURE (new);
+  return FROGR_PICTURE (g_object_new(FROGR_TYPE_PICTURE,
+                                     "fileuri", fileuri,
+                                     "title", title,
+                                     "is-public", public,
+                                     "is-family", family,
+                                     "is-friend", friend,
+                                     "safety-level", FSP_SAFETY_LEVEL_SAFE,
+                                     "content-type", FSP_CONTENT_TYPE_PHOTO,
+                                     "show_in_search", TRUE,
+                                     NULL));
 }
 
 
@@ -503,11 +539,11 @@ frogr_picture_new (const gchar *filepath,
 const gchar *
 frogr_picture_get_id (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return (const gchar *)priv->id;
 }
 
@@ -515,47 +551,47 @@ void
 frogr_picture_set_id (FrogrPicture *self,
                       const gchar *id)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   g_free (priv->id);
   priv->id = g_strdup (id);
 }
 
 const gchar *
-frogr_picture_get_filepath (FrogrPicture *self)
+frogr_picture_get_fileuri (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
-  return (const gchar *)priv->filepath;
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  return (const gchar *)priv->fileuri;
 }
 
 void
-frogr_picture_set_filepath (FrogrPicture *self,
-                            const gchar *filepath)
+frogr_picture_set_fileuri (FrogrPicture *self,
+                           const gchar *fileuri)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
-  g_free (priv->filepath);
-  priv->filepath = g_strdup (filepath);
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  g_free (priv->fileuri);
+  priv->fileuri = g_strdup (fileuri);
 }
 
 const gchar *
 frogr_picture_get_title (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return (const gchar *)priv->title;
 }
 
@@ -563,12 +599,12 @@ void
 frogr_picture_set_title (FrogrPicture *self,
                          const gchar *title)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
   g_return_if_fail(title != NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   g_free (priv->title);
   priv->title = g_strdup (title);
 }
@@ -576,11 +612,11 @@ frogr_picture_set_title (FrogrPicture *self,
 const gchar *
 frogr_picture_get_description (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return (const gchar *)priv->description;
 }
 
@@ -588,11 +624,11 @@ void
 frogr_picture_set_description (FrogrPicture *self,
                                const gchar *description)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   g_free (priv->description);
   priv->description = g_strdup (description);
 }
@@ -600,32 +636,33 @@ frogr_picture_set_description (FrogrPicture *self,
 const GSList *
 frogr_picture_get_tags_list (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->tags_list;
 }
 
 const gchar *
 frogr_picture_get_tags (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->tags_string;
 }
 
 void
 frogr_picture_set_tags (FrogrPicture *self, const gchar *tags_string)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
 
   /* First remove all the previous tags list */
   g_slist_foreach (priv->tags_list, (GFunc) g_free, NULL);
@@ -641,7 +678,6 @@ frogr_picture_add_tags (FrogrPicture *self, const gchar *tags_string)
 {
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  /* Add to internal tags_list */
   _add_tags_to_tags_list (self, tags_string);
 }
 
@@ -656,11 +692,11 @@ frogr_picture_remove_tags (FrogrPicture *self)
 gboolean
 frogr_picture_is_public (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->is_public;
 }
 
@@ -668,22 +704,22 @@ void
 frogr_picture_set_public (FrogrPicture *self,
                           gboolean public)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->is_public = public;
 }
 
 gboolean
 frogr_picture_is_friend (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->is_friend;
 }
 
@@ -691,22 +727,22 @@ void
 frogr_picture_set_friend (FrogrPicture *self,
                           gboolean friend)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->is_friend = friend;
 }
 
 gboolean
 frogr_picture_is_family (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->is_family;
 }
 
@@ -714,22 +750,22 @@ void
 frogr_picture_set_family (FrogrPicture *self,
                           gboolean family)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->is_family = family;
 }
 
 FspSafetyLevel
 frogr_picture_get_safety_level (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->safety_level;
 }
 
@@ -737,22 +773,22 @@ void
 frogr_picture_set_safety_level (FrogrPicture *self,
                                 FspSafetyLevel safety_level)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->safety_level = safety_level;
 }
 
 FspContentType
 frogr_picture_get_content_type (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->content_type;
 }
 
@@ -760,22 +796,22 @@ void
 frogr_picture_set_content_type (FrogrPicture *self,
                                 FspContentType content_type)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->content_type = content_type;
 }
 
 gboolean
 frogr_picture_show_in_search (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->show_in_search;
 }
 
@@ -783,22 +819,22 @@ void
 frogr_picture_set_show_in_search (FrogrPicture *self,
                                   gboolean show_in_search)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   priv->show_in_search = show_in_search;
 }
 
 GdkPixbuf *
 frogr_picture_get_pixbuf (FrogrPicture *self)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->pixbuf;
 }
 
@@ -806,24 +842,67 @@ void
 frogr_picture_set_pixbuf (FrogrPicture *self,
                           GdkPixbuf *pixbuf)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
 
-  FrogrPicturePrivate *priv =
-    FROGR_PICTURE_GET_PRIVATE (self);
-
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
   if (priv->pixbuf)
     g_object_unref (priv->pixbuf);
 
   priv->pixbuf = GDK_IS_PIXBUF (pixbuf) ? g_object_ref (pixbuf) : NULL;
 }
 
+gulong frogr_picture_get_filesize (FrogrPicture *self)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), 0);
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  return priv->filesize;
+}
+
+void frogr_picture_set_filesize (FrogrPicture *self, gulong filesize)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_PICTURE(self));
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  priv->filesize = filesize;
+}
+
+void
+frogr_picture_set_datetime (FrogrPicture *self, const gchar *datetime)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_if_fail(FROGR_IS_PICTURE(self));
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  g_free (priv->datetime);
+  priv->datetime = g_strdup (datetime);
+}
+
+const gchar *
+frogr_picture_get_datetime (FrogrPicture *self)
+{
+  FrogrPicturePrivate *priv = NULL;
+
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), 0);
+
+  priv = FROGR_PICTURE_GET_PRIVATE (self);
+  return (const gchar *)priv->datetime;
+}
+
 GSList *
 frogr_picture_get_sets (FrogrPicture *self)
 {
-  g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
-
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->sets;
 }
@@ -831,13 +910,13 @@ frogr_picture_get_sets (FrogrPicture *self)
 void
 frogr_picture_set_sets (FrogrPicture *self, GSList *sets)
 {
-  g_return_if_fail(FROGR_IS_PICTURE(self));
-
   GSList *new_list = NULL;
   GSList *item = NULL;
   FrogrPhotoSet *set = NULL;
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_PICTURE(self));
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
 
   /* First remove all the previous sets list */
@@ -857,11 +936,11 @@ frogr_picture_set_sets (FrogrPicture *self, GSList *sets)
 void
 frogr_picture_add_set (FrogrPicture *self, FrogrPhotoSet *set)
 {
+  FrogrPicturePrivate *priv = NULL;
+
   g_return_if_fail(FROGR_IS_PICTURE(self));
   g_return_if_fail(FROGR_IS_SET(set));
 
-  FrogrPicturePrivate *priv = NULL;
-
   /* Do not add the same set twice */
   if (frogr_picture_in_set (self, set))
     return;
@@ -881,10 +960,10 @@ frogr_picture_remove_sets (FrogrPicture *self)
 gboolean
 frogr_picture_in_set (FrogrPicture *self, FrogrPhotoSet *set)
 {
-  g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
-
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
   if (g_slist_index (priv->sets, set) != -1)
     return TRUE;
@@ -895,10 +974,10 @@ frogr_picture_in_set (FrogrPicture *self, FrogrPhotoSet *set)
 GSList *
 frogr_picture_get_groups (FrogrPicture *self)
 {
-  g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
-
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), NULL);
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
   return priv->groups;
 }
@@ -906,13 +985,13 @@ frogr_picture_get_groups (FrogrPicture *self)
 void
 frogr_picture_set_groups (FrogrPicture *self, GSList *groups)
 {
-  g_return_if_fail(FROGR_IS_PICTURE(self));
-
   GSList *new_list = NULL;
   GSList *item = NULL;
   FrogrGroup *group = NULL;
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_if_fail(FROGR_IS_PICTURE(self));
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
 
   /* First remove all the previous groups list */
@@ -940,10 +1019,10 @@ frogr_picture_remove_groups (FrogrPicture *self)
 gboolean
 frogr_picture_in_group (FrogrPicture *self, FrogrGroup *group)
 {
-  g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
-
   FrogrPicturePrivate *priv = NULL;
 
+  g_return_val_if_fail(FROGR_IS_PICTURE(self), FALSE);
+
   priv = FROGR_PICTURE_GET_PRIVATE (self);
   if (g_slist_index (priv->groups, group) != -1)
     return TRUE;
