@@ -1,7 +1,7 @@
 /*
  * frogr-picture.h -- A picture in frogr
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -57,7 +57,7 @@ struct _FrogrPictureClass
 GType frogr_picture_get_type(void) G_GNUC_CONST;
 
 /* Constructor */
-FrogrPicture *frogr_picture_new (const gchar *filepath,
+FrogrPicture *frogr_picture_new (const gchar *fileuri,
                                  const gchar *title,
                                  gboolean public,
                                  gboolean family,
@@ -77,9 +77,9 @@ const gchar *frogr_picture_get_description (FrogrPicture *self);
 void frogr_picture_set_description (FrogrPicture *self,
                                     const gchar *description);
 
-const gchar *frogr_picture_get_filepath (FrogrPicture *self);
-void frogr_picture_set_filepath (FrogrPicture *self,
-                                 const gchar *filepath);
+const gchar *frogr_picture_get_fileuri (FrogrPicture *self);
+void frogr_picture_set_fileuri (FrogrPicture *self,
+                                const gchar *fileuri);
 
 const GSList *frogr_picture_get_tags_list (FrogrPicture *self);
 const gchar *frogr_picture_get_tags (FrogrPicture *self);
@@ -113,6 +113,12 @@ GdkPixbuf *frogr_picture_get_pixbuf (FrogrPicture *self);
 void frogr_picture_set_pixbuf (FrogrPicture *self,
                                GdkPixbuf *pixbuf);
 
+gulong frogr_picture_get_filesize (FrogrPicture *self);
+void frogr_picture_set_filesize (FrogrPicture *self, gulong filesize);
+
+void frogr_picture_set_datetime (FrogrPicture *self, const gchar *datetime);
+const gchar *frogr_picture_get_datetime (FrogrPicture *self);
+
 GSList *frogr_picture_get_sets (FrogrPicture *self);
 void frogr_picture_set_sets (FrogrPicture *self, GSList *sets);
 void frogr_picture_add_set (FrogrPicture *self, FrogrPhotoSet *set);
