@@ -1,7 +1,7 @@
 /*
  * frogr-settings-dialog.c -- Main settings dialog
  *
- * Copyright (C) 2009, 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2009-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -36,7 +36,7 @@
                                 FROGR_TYPE_SETTINGS_DIALOG,      \
                                 FrogrSettingsDialogPrivate))
 
-G_DEFINE_TYPE (FrogrSettingsDialog, frogr_settings_dialog, GTK_TYPE_DIALOG);
+G_DEFINE_TYPE (FrogrSettingsDialog, frogr_settings_dialog, GTK_TYPE_DIALOG)
 
 typedef struct _FrogrSettingsDialogPrivate {
   FrogrController *controller;
@@ -53,6 +53,8 @@ typedef struct _FrogrSettingsDialogPrivate {
   GtkWidget *safe_rb;
   GtkWidget *moderate_rb;
   GtkWidget *restricted_rb;
+  GtkWidget *disable_tags_autocompletion_cb;
+  GtkWidget *keep_file_extensions_cb;
 
   GtkWidget *use_proxy_cb;
   GtkWidget *proxy_host_label;
@@ -68,10 +70,12 @@ typedef struct _FrogrSettingsDialogPrivate {
   gboolean family_visibility;
   gboolean friend_visibility;
   gboolean show_in_search;
+  gboolean disable_tags_autocompletion;
+  gboolean keep_file_extensions;
   FspSafetyLevel safety_level;
   FspContentType content_type;
 
-  gboolean use_proxy;;
+  gboolean use_proxy;
   gchar *proxy_host;
   gchar *proxy_port;
   gchar *proxy_username;
@@ -83,6 +87,10 @@ static FrogrSettingsDialog *_instance = NULL;
 
 /* Prototypes */
 
+static void _add_togleabble_item (FrogrSettingsDialog *self, GtkBox *box,
+                                  GtkRadioButton *radio_member, gboolean force_radio,
+                                  const gchar *mnemonic, GtkWidget **out_ref);
+
 static void _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook);
 
 static void _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook);
@@ -106,6 +114,27 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
 
 /* Private API */
 
+static void
+_add_togleabble_item (FrogrSettingsDialog *self, GtkBox *box,
+                      GtkRadioButton *radio_member, gboolean force_radio,
+                      const gchar *mnemonic, GtkWidget **out_ref)
+{
+  GtkWidget *item = NULL;
+
+  /* If it has a group associated, it's a radio button */
+  if (radio_member || force_radio)
+    item = gtk_radio_button_new_with_mnemonic_from_widget (radio_member, mnemonic);
+  else
+    item = gtk_check_button_new_with_mnemonic (mnemonic);
+
+  g_signal_connect (G_OBJECT (item), "toggled",
+                    G_CALLBACK (_on_button_toggled),
+                    self);
+
+  gtk_box_pack_start (box, item, FALSE, FALSE, 0);
+  *out_ref = item;
+}
+
 static void
 _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 {
@@ -116,8 +145,6 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   GtkWidget *padding_hbox = NULL;
   GtkWidget *align = NULL;
   GtkWidget *label = NULL;
-  GtkWidget *rbutton = NULL;
-  GtkWidget *cbutton = NULL;
   gchar *markup = NULL;
 
   priv = FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
@@ -139,52 +166,29 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   box1 = gtk_vbox_new (FALSE, 6);
   box2 = gtk_hbox_new (FALSE, 12);
 
-  rbutton = gtk_radio_button_new_with_mnemonic (NULL, _("_Private"));
-  gtk_box_pack_start (GTK_BOX (box2), rbutton, FALSE, FALSE, 0);
-  priv->private_rb = rbutton;
-
-  rbutton = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (priv->private_rb), _("P_ublic"));
-  gtk_box_pack_start (GTK_BOX (box2), rbutton, FALSE, FALSE, 0);
-  priv->public_rb = rbutton;
+  _add_togleabble_item (self, GTK_BOX (box2), NULL, TRUE, _("_Private"), &priv->private_rb);
+  _add_togleabble_item (self, GTK_BOX (box2), GTK_RADIO_BUTTON (priv->private_rb),
+                        FALSE, _("P_ublic"), &priv->public_rb);
 
   gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, FALSE, 0);
 
   box2 = gtk_vbox_new (FALSE, 6);
 
-  cbutton = gtk_check_button_new_with_mnemonic (_("Visible to _Family"));
-  gtk_box_pack_start (GTK_BOX (box2), cbutton, FALSE, FALSE, 0);
-  priv->family_cb = cbutton;
-
-  cbutton = gtk_check_button_new_with_mnemonic (_("Visible to F_riends"));
-  gtk_box_pack_start (GTK_BOX (box2), cbutton, FALSE, FALSE, 0);
-  priv->friend_cb = cbutton;
+  _add_togleabble_item (self, GTK_BOX (box2), NULL, FALSE,
+                        _("Visible to _Family"), &priv->family_cb);
+  _add_togleabble_item (self, GTK_BOX (box2), NULL, FALSE,
+                        _("Visible to F_riends"), &priv->friend_cb);
 
   padding_hbox = gtk_hbox_new (FALSE, 0);
   gtk_box_pack_start (GTK_BOX (padding_hbox), box2, FALSE, FALSE, 12);
   gtk_box_pack_start (GTK_BOX (box1), padding_hbox, FALSE, FALSE, 0);
 
-  cbutton = gtk_check_button_new_with_mnemonic (_("_Show up in Global Search Results"));
-  gtk_box_pack_start (GTK_BOX (box1), cbutton, FALSE, FALSE, 0);
-  priv->show_in_search_cb = cbutton;
+  _add_togleabble_item (self, GTK_BOX (box1), NULL, FALSE,
+                        _("_Show up in Global Search Results"),
+                        &priv->show_in_search_cb);
 
   gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
-  g_signal_connect (G_OBJECT (priv->public_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
-  g_signal_connect (G_OBJECT (priv->family_cb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
-  g_signal_connect (G_OBJECT (priv->friend_cb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
-  g_signal_connect (G_OBJECT (priv->show_in_search_cb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
   /* Default Content type */
 
   label = gtk_label_new (NULL);
@@ -200,32 +204,15 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   box1 = gtk_hbox_new (FALSE, 12);
 
-  rbutton = gtk_radio_button_new_with_mnemonic (NULL, _("P_hoto"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->photo_content_rb = rbutton;
-
-  rbutton = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (priv->photo_content_rb), _("Scree_nshot"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->sshot_content_rb = rbutton;
-
-  rbutton = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (priv->photo_content_rb), _("Oth_er"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->other_content_rb = rbutton;
+  _add_togleabble_item (self, GTK_BOX (box1), NULL, TRUE,
+                        _("P_hoto"), &priv->photo_content_rb);
+  _add_togleabble_item (self, GTK_BOX (box1), GTK_RADIO_BUTTON (priv->photo_content_rb),
+                        FALSE, _("Scree_nshot"), &priv->sshot_content_rb);
+  _add_togleabble_item (self, GTK_BOX (box1), GTK_RADIO_BUTTON (priv->photo_content_rb),
+                        FALSE, _("Oth_er"), &priv->other_content_rb);
 
   gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
-  g_signal_connect (G_OBJECT (priv->photo_content_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
-  g_signal_connect (G_OBJECT (priv->sshot_content_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
-  g_signal_connect (G_OBJECT (priv->other_content_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
-
   /* Default Safety level */
 
   label = gtk_label_new (NULL);
@@ -241,31 +228,38 @@ _add_general_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
 
   box1 = gtk_hbox_new (FALSE, 12);
 
-  rbutton = gtk_radio_button_new_with_mnemonic (NULL, _("S_afe"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->safe_rb = rbutton;
-
-  rbutton = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (priv->safe_rb), _("_Moderate"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->moderate_rb = rbutton;
-
-  rbutton = gtk_radio_button_new_with_mnemonic_from_widget (GTK_RADIO_BUTTON (priv->safe_rb), _("Restr_icted"));
-  gtk_box_pack_start (GTK_BOX (box1), rbutton, FALSE, FALSE, 0);
-  priv->restricted_rb = rbutton;
+  _add_togleabble_item (self, GTK_BOX (box1), NULL, TRUE,
+                        _("S_afe"), &priv->safe_rb);
+  _add_togleabble_item (self, GTK_BOX (box1), GTK_RADIO_BUTTON (priv->safe_rb),
+                        FALSE, _("_Moderate"), &priv->moderate_rb);
+  _add_togleabble_item (self, GTK_BOX (box1), GTK_RADIO_BUTTON (priv->safe_rb),
+                        FALSE, _("Restr_icted"), &priv->restricted_rb);
 
   gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
-  g_signal_connect (G_OBJECT (priv->safe_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
+  /* Misc */
 
-  g_signal_connect (G_OBJECT (priv->moderate_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
+  label = gtk_label_new (NULL);
+  gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+  markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+                                    _("Other options"));
+  gtk_label_set_markup (GTK_LABEL (label), markup);
+  g_free (markup);
 
-  g_signal_connect (G_OBJECT (priv->restricted_rb), "toggled",
-                    G_CALLBACK (_on_button_toggled),
-                    self);
+  align = gtk_alignment_new (0, 0, 0, 1);
+  gtk_container_add (GTK_CONTAINER (align), label);
+  gtk_box_pack_start (GTK_BOX (vbox), align, FALSE, FALSE, 6);
+
+  box1 = gtk_vbox_new (FALSE, 6);
+
+  _add_togleabble_item (self, GTK_BOX (box1), NULL, FALSE,
+                        _("Disa_ble Tags Auto-Completion"),
+                        &priv->disable_tags_autocompletion_cb);
+  _add_togleabble_item (self, GTK_BOX (box1), NULL, FALSE,
+                        _("_Keep File Extensions in Titles when Loading Pictures"),
+                        &priv->keep_file_extensions_cb);
+
+  gtk_box_pack_start (GTK_BOX (vbox), box1, FALSE, FALSE, 0);
 
   gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
   gtk_notebook_append_page (notebook, vbox, gtk_label_new_with_mnemonic (_("_General")));
@@ -299,10 +293,10 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 0);
 
   label = gtk_label_new_with_mnemonic (_("_Host:"));
-  align = gtk_alignment_new (1, 0, 1, 0);
+  align = gtk_alignment_new (0, 0, 0, 0);
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_table_attach (GTK_TABLE (table), align, 0, 1, 0, 1,
-                    0, 0, 6, 6);
+                    GTK_FILL, 0, 6, 6);
   priv->proxy_host_label = label;
 
   entry = gtk_entry_new ();
@@ -320,7 +314,7 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   align = gtk_alignment_new (0, 0, 0, 0);
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_table_attach (GTK_TABLE (table), align, 0, 1, 1, 2,
-                    0, 0, 6, 6);
+                    GTK_FILL, 0, 6, 6);
   priv->proxy_port_label = label;
 
   entry = gtk_entry_new ();
@@ -338,7 +332,7 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   align = gtk_alignment_new (0, 0, 0, 0);
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_table_attach (GTK_TABLE (table), align, 0, 1, 2, 3,
-                    0, 0, 6, 6);
+                    GTK_FILL, 0, 6, 6);
   priv->proxy_username_label = label;
 
   entry = gtk_entry_new ();
@@ -356,7 +350,7 @@ _add_connection_page (FrogrSettingsDialog *self, GtkNotebook *notebook)
   align = gtk_alignment_new (0, 0, 0, 0);
   gtk_container_add (GTK_CONTAINER (align), label);
   gtk_table_attach (GTK_TABLE (table), align, 0, 1, 3, 4,
-                    0, 0, 6, 6);
+                    GTK_FILL, 0, 6, 6);
   priv->proxy_password_label = label;
 
   entry = gtk_entry_new ();
@@ -388,14 +382,15 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
   FrogrSettingsDialogPrivate *priv =
     FROGR_SETTINGS_DIALOG_GET_PRIVATE (self);
 
-  /* Get data form configuration */
-
+  /* Get data from configuration */
   priv->public_visibility = frogr_config_get_default_public (priv->config);
   priv->family_visibility = frogr_config_get_default_family (priv->config);
   priv->friend_visibility = frogr_config_get_default_friend (priv->config);
   priv->show_in_search = frogr_config_get_default_show_in_search (priv->config);
   priv->content_type = frogr_config_get_default_content_type (priv->config);
   priv->safety_level = frogr_config_get_default_safety_level (priv->config);
+  priv->disable_tags_autocompletion = !frogr_config_get_tags_autocompletion (priv->config);
+  priv->keep_file_extensions = frogr_config_get_keep_file_extensions (priv->config);
   priv->use_proxy = frogr_config_get_use_proxy (priv->config);
 
   g_free (priv->proxy_host);
@@ -445,6 +440,12 @@ _fill_dialog_with_data (FrogrSettingsDialog *self)
   else
     gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->safe_rb), TRUE);
 
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->disable_tags_autocompletion_cb),
+                                priv->disable_tags_autocompletion);
+
+  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->keep_file_extensions_cb),
+                                priv->keep_file_extensions);
+
   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (priv->use_proxy_cb),
                                 priv->use_proxy);
   if (priv->proxy_host)
@@ -478,6 +479,9 @@ _save_data (FrogrSettingsDialog *self)
   frogr_config_set_default_content_type (priv->config, priv->content_type);
   frogr_config_set_default_safety_level (priv->config, priv->safety_level);
 
+  frogr_config_set_tags_autocompletion (priv->config, !priv->disable_tags_autocompletion);
+  frogr_config_set_keep_file_extensions (priv->config, priv->keep_file_extensions);
+
   frogr_config_set_use_proxy (priv->config, priv->use_proxy);
 
   g_free (priv->proxy_host);
@@ -602,6 +606,18 @@ _on_button_toggled (GtkToggleButton *button, gpointer data)
       DEBUG ("Content type set to %d", priv->safety_level);
     }
 
+  if (GTK_WIDGET (button) == priv->disable_tags_autocompletion_cb)
+    {
+      priv->disable_tags_autocompletion = active;
+      DEBUG ("Disable tags autocompletion set to %s", active ? "TRUE" : "FALSE");
+    }
+
+  if (GTK_WIDGET (button) == priv->keep_file_extensions_cb)
+    {
+      priv->keep_file_extensions = active;
+      DEBUG ("Keep file extensions in title set to %s", active ? "TRUE" : "FALSE");
+    }
+
   if (GTK_WIDGET (button) == priv->use_proxy_cb)
     {
       priv->use_proxy = active;
@@ -648,6 +664,10 @@ static void _dialog_response_cb (GtkDialog *dialog, gint response, gpointer data
   if (response == GTK_RESPONSE_OK && _save_data (self) == FALSE)
       return;
 
+  /* Fetch tags if needed */
+  if (!priv->disable_tags_autocompletion)
+    frogr_controller_fetch_tags_if_needed (priv->controller);
+
   /* Update proxy status */
   if (priv->use_proxy)
     frogr_controller_set_proxy (priv->controller,
@@ -724,6 +744,8 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->safe_rb = NULL;
   priv->moderate_rb = NULL;
   priv->restricted_rb = NULL;
+  priv->disable_tags_autocompletion_cb = NULL;
+  priv->keep_file_extensions_cb = NULL;
   priv->use_proxy_cb = NULL;
   priv->proxy_host_label = NULL;
   priv->proxy_host_entry = NULL;
@@ -739,6 +761,8 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
   priv->show_in_search = FALSE;
   priv->safety_level = FSP_SAFETY_LEVEL_NONE;
   priv->content_type = FSP_CONTENT_TYPE_NONE;
+  priv->disable_tags_autocompletion = FALSE;
+  priv->keep_file_extensions = FALSE;
   priv->use_proxy = FALSE;
   priv->proxy_host = NULL;
   priv->proxy_port = NULL;
@@ -771,8 +795,6 @@ frogr_settings_dialog_init (FrogrSettingsDialog *self)
                     G_CALLBACK (_on_dialog_delete_event),
                     NULL);
 
-  /* Show the UI */
-  gtk_widget_show_all (GTK_WIDGET (self));
   gtk_dialog_set_default_response (GTK_DIALOG (self),
                                    GTK_RESPONSE_OK);
 }
