@@ -1,7 +1,7 @@
 /*
  * frogr-util.h -- Misc tools.
  *
- * Copyright (C) 2010, 2011 Mario Sanchez Prada
+ * Copyright (C) 2010-2011 Mario Sanchez Prada
  * Authors: Mario Sanchez Prada <msanchez@igalia.com>
  *
  * This program is free software; you can redistribute it and/or
@@ -25,6 +25,12 @@
 
 G_BEGIN_DECLS
 
+const gchar *frogr_util_get_app_data_dir (void);
+
+const gchar *frogr_util_get_icons_dir (void);
+
+const gchar *frogr_util_get_locale_dir (void);
+
 void frogr_util_open_url_in_browser (const gchar *url);
 
 void frogr_util_show_info_dialog (GtkWindow *parent, const gchar *message);
